/* 
 * Copyright (c) 2004 James Bailie <jimmy@jamesbailie.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *     * The name of James Bailie may not be used to endorse or promote 
 * products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include<errno.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<sys/types.h>
#include<sys/socket.h>
#include<netinet/in.h>
#include<unistd.h>
#include<netdb.h>

int tcp_connect( char *hostname, int port )
{
   struct sockaddr_in serv_addr;
   struct hostent *host;
   int fd;

   bzero( &serv_addr, sizeof( serv_addr ));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_port = htons( port );

   host = gethostbyname( hostname );

   if ( host == NULL )
   {
      fprintf( stderr, "httpf: gethostbyname: %s.\n", hstrerror( h_errno ));
      exit( 1 );
   }

   bcopy( *host->h_addr_list, &serv_addr.sin_addr, sizeof( struct in_addr ));

   fd = socket( PF_INET, SOCK_STREAM, 0 );

   if ( socket < 0 )
   {
      fprintf( stderr, "httpf: socket: %s.\n", strerror( errno ));
      exit( 1 );
   }

   if ( connect( fd, ( struct sockaddr *)&serv_addr, sizeof( serv_addr )) < 0 )
   {
      fprintf( stderr, "httpf: connect: %s.\n", strerror( errno ));
      exit( 1 );
   }

   return fd;
}

void transfer( int fd )
{
   char in_buffer[ 512 ], out_buffer[ 4096 ], *in_ptr, *out_ptr;
   int i, result, in_len, out_len;
   fd_set in_set, out_set;

   in_ptr = in_buffer;
   out_ptr = out_buffer;

   *in_ptr = *out_ptr = '\0';
   in_len = out_len = 0;

   i = fd + 1;

AGAIN:
   FD_ZERO( &in_set );
   FD_ZERO( &out_set );

   FD_SET( fd, &in_set );
   FD_SET( fd, &out_set );

   FD_SET( STDIN_FILENO, &in_set );
   FD_SET( STDOUT_FILENO, &out_set );

   result = select( i, &in_set, &out_set, NULL, NULL );

   if ( result < 0 )
   {
      fprintf( stderr, "httpf: select: %s.\n", strerror( errno ));
      exit( 1 );
   }

   if ( in_ptr != NULL && in_len == 0 && FD_ISSET( STDIN_FILENO, &in_set ))
   {
      result = read( STDIN_FILENO, in_buffer, sizeof( in_buffer ) - 1 );

      if ( result < 0 )
      {
         fprintf( stderr, "httpf: read: %s.\n", strerror( errno ));
         exit( 1 );
      }

      if ( read == 0 )
         in_ptr = NULL;
      else
      {
         in_len = result;
         in_buffer[ in_len ] = '\0';
         in_ptr = in_buffer;
      }
   }

   if ( in_ptr != NULL && in_len && FD_ISSET( fd, &out_set ))
   {
      result = write( fd, in_ptr, in_len );

      if ( result < 0 )
      {
         fprintf( stderr, "httpf: write: %s.\n", strerror( errno ));
         exit( 1 );
      }

      in_ptr += result;
      in_len -= result;
   }

   if ( out_len == 0 && FD_ISSET( fd, &in_set ))
   {
      result = read( fd, out_buffer, sizeof( out_buffer ) - 1 );

      if ( result < 0 )
      {
         fprintf( stderr, "httpf: read: %s.\n", strerror( errno ));
         exit( 1 );
      }

      if ( result == 0 )
         return;
      else
      {
         out_len = result;
         out_buffer[ out_len ] = '\0';
         out_ptr = out_buffer;
      }
   }

   if ( out_len && FD_ISSET( STDOUT_FILENO, &out_set ))
   {
      result = write( STDOUT_FILENO, out_ptr, out_len );

      if ( result < 0 )
      {
         fprintf( stderr, "httpf: write: %s.\n", strerror( errno ));
         exit( 1 );
      }

      out_ptr += result;
      out_len -= result;
   }

   goto AGAIN;
}

int main( int argc, char **argv )
{
   if ( argc < 2 )
   {
      fprintf( stderr, "httpf: hostname unspecified.\n" );
      exit( 1 );
   }

   transfer( tcp_connect( argv[ 1 ], ( argc < 3 ? 80 : atoi( argv[ 2 ] ))));

   return 0;
}
