/*
 * Copyright (c) 2001, 2002, 2003, 2004 James Bailie <jimmy@jamesbailie.com>.  
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *     * The name of James Bailie may not be used to endorse or promote 
 * products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

int compare_numbers( const void *a, const void *b )
{
   if ( **(( double **)a ) == **(( double **)b ) )
      return 0;
   else if ( **(( double **)a ) > **(( double **)b ) )
      return 1;
   else
      return -1;
}

int compare_car_numbers( const void *a, const void *b )
{
   if ( *( *( struct object **)a )->data.head->data.atom->data.value ==
        *( *( struct object **)b )->data.head->data.atom->data.value )
      return 0;
   else if ( *( *( struct object **)a )->data.head->data.atom->data.value >
             *( *( struct object **)b )->data.head->data.atom->data.value )
      return 1;
   else
      return -1;
}

int compare_car_strings( const void *a, const void *b )
{
   return strncasecmp( 
      ( *( struct object **)a )->data.head->data.atom->data.string->string,
      ( *( struct object **)b )->data.head->data.atom->data.string->string,
      MIN ( 
         ( *( struct object **)a )->data.head->data.atom->data.string->length,
         ( *( struct object **)b )->data.head->data.atom->data.string->length ) );
}

int compare_strings( const void *a, const void *b )
{
   return strncasecmp( ( *( struct lstring **)a )->string, 
                       ( *( struct lstring **)b )->string,
                       MIN( ( *( struct lstring **)a )->length,
                            ( *( struct lstring **)b )->length ));
}

char *expand_tilde( char *name )
{
   struct string *s;
   char *ptr, *ptr2;

   s = make_string();

   ptr = name;
   while( *ptr )
      string_append( s, *ptr++ );

   if (( s->used == 1 && s->str[ 0 ] == '~' ) ||
       ( s->used >= 2 && s->str[ 1 ] == '/' ))
   {
      string_erase( s, 0 );

      ptr = getenv( "HOME" );

      if ( ptr == NULL )
         ptr = str_dup( name, strlen( name ));
      else
      {
         ptr2 = ptr;
         ptr = &ptr[ strlen( ptr ) ];

         while( ptr > ptr2 )
            string_prepend( s, *--ptr );

         ptr = str_dup( s->str, s->used );
      }
   }
   else if ( s->used > 2 && s->str[ 0 ] == '~' && isalnum( s->str[ 1 ] ))
   {
      struct string *login;
      struct passwd *passwd;

      login = make_string();
      ptr = &s->str[ 1 ];

      while( *ptr && *ptr != '/' )
         string_append( login, *ptr++ );

      passwd = getpwnam( login->str );

      if ( passwd == NULL )
         ptr = str_dup( name, strlen( name ));
      else
      {
         string_assign( login, passwd->pw_dir, strlen( passwd->pw_dir ));

         while( *ptr )
            string_append( login, *ptr++ );

         ptr = str_dup( login->str, login->used );
      }

      string_free( login );
   }
   else
      ptr = str_dup( name, strlen( name ));

   string_free( s );

   return ptr;
}

struct stack *find_possibilities( char *name )
{
   char *d, *e, *w, *ptr;
   DIR *dir, *o;
   struct dirent *dp;
   struct stack *entries;
   struct string *s, *t;
   int length, result;

   regmatch_t matches[ 4 ];

   result = regexec( &find_poss_regex, name, 4, matches, 0 );

   if ( result )
   {
      char err[ 80 ];

      free( name );
      regerror( result, &find_poss_regex, err, sizeof( err ));
      fprintf( stderr, "find_possibilities: regexec(): %s\n", err );
      return NULL;
   }

   length = matches[ 0 ].rm_eo - matches[ 0 ].rm_so;
   w = memory( length + 1 );
   bcopy( &name[ matches[ 0 ].rm_so ], w, length );
   w[ length ] = '\0';

   length = matches[ 1 ].rm_eo - matches[ 1 ].rm_so;
   d = memory( length + 1 );
   bcopy( &name[ matches[ 1 ].rm_so ], d, length );
   d[ length ] = '\0';

   length = matches[ 3 ].rm_eo - matches[ 3 ].rm_so;
   e = memory( length + 1 );
   bcopy( &name[ matches[ 3 ].rm_so ], e, length );
   e[ length ] = '\0';

   entries = make_stack();
   dir = opendir(( *d == '\0' ? "." : d ));

   t = make_string();

   if ( dir != NULL )
   {
      length = strlen( e );

      while(( dp = readdir( dir )) != NULL )
      {
         if ( dp->d_namlen >= length &&
              strncmp( e, dp->d_name, length ) == 0 )
         {
            s = make_string();
            string_assign( s, dp->d_name, dp->d_namlen );
            stack_push( entries, s );

            string_assign( t, d, strlen( d ));
            if ( t->used )
               string_append( t, '/' );

            ptr = dp->d_name;
            while( *ptr )
               string_append( t, *ptr++ );

            if (( o = opendir( t->str )) != NULL )
            {
               closedir( o );
               string_append( s, '/' );
            }
         }
      }

      closedir( dir );
   }
   else
   {
      dir = opendir( w );

      if ( dir != NULL )
      {
         while(( dp = readdir( dir )) != NULL )
         {
            s = make_string();
            string_assign( s, dp->d_name, dp->d_namlen );
            stack_push( entries, s );

            string_assign( t, w, strlen( w ));
            string_append( t, '/' );

            ptr = dp->d_name;
            while( *ptr )
               string_append( t, *ptr++ );

            if (( o = opendir( t->str )) != NULL )
            {
               closedir( o );
               string_append( s, '/' );
            }

         }

         closedir( dir );
      }
   }

   string_free( t );

   free( w );
   free( d );
   free( e );

   return entries;
}

struct stack *format_possibilities( struct stack *entries )
{
   int i, max = 1, cols, rows, total, column, target, width;
   char mask[ 10 ], *line, *ptr;
   struct stack *formatted;
   struct string *s, *t;

   for( i = 0; i < entries->used; ++i )
   {
      struct string *s;

      s = ( struct string *)entries->values[ i ];
      if ( s->used > max )
         max = s->used;
   }

   ++max;

   do_cols( "format_possibilities", NULL );
   width = *( *( struct object **)stack->top )->data.atom->data.value;
   stack_pop( stack );

   line = memory( width + 1 );

   cols = --width / max;
   if ( cols == 0 )
      cols = 1;

   rows = ( entries->used + cols ) / cols;
   total = rows * cols;

   snprintf( mask, sizeof( mask ), "%%-%ds ", max - 1 );

   formatted = make_stack();
   t = make_string();

   for( i = 0; i < total; ++i )
   {
      column = i % cols;
      target = column * rows + i / cols;

      if ( target < entries->used )
      {
         s = ( struct string *)entries->values[ target ];
         snprintf( line, width, mask, s->str );
         ptr = line;
         while( *ptr )
            string_append( t, *ptr++ );
      }

      if ( column == cols - 1 )
      {
         while( t->used < width )
            string_append( t, ' ' );

         stack_push( formatted, t );
         t = make_string();
      }
   }

   string_free( t );
   free( line );

   return formatted;
}

struct string *find_common_prefix( struct stack *entries )
{
   struct stack *e2 = NULL;
   struct string *s, *t, *p;

   if ( entries->used == 0 )
      return NULL;

   p = make_string();

   if ( entries->used == 1 )
   {
      s = ( struct string *)entries->values[ 0 ];
      string_assign( p, s->str, s->used );
      return p;
   }

   {
      int i, j, k, min = 10000;

      for( i = 0; i < 2; ++i )
      {
         for( j = 0; j < entries->used; ++j )
         {
            s = ( struct string *)entries->values[ j ];
            if ( s->used < min )
               min = s->used;
         }

         s = ( struct string *)entries->values[ 0 ];

         for( j = 0; j < min; ++j )
         {
            for( k = 1; k < entries->used; ++k )
            {
               t = ( struct string *)entries->values[ k ];

               if ( s->str[ j ] != t->str[ j ] )
                  goto NEXT;
            }

            string_append( p, s->str[ j ] );
         }

      NEXT:
         if ( p->used || i == 1 )
            break;

         e2 = make_stack();

         for( j = 0; j < entries->used; ++j )
         {
            s = ( struct string *)entries->values[ j ];
            if ( strcmp( s->str, "../" ) && strcmp( s->str, "./" ))
               stack_push( e2, s );
         }

         if ( e2->used == 1 )
         {
            s = ( struct string *)e2->values[ 0 ];
            string_assign( p, s->str, s->used );
            break;
         }

         min = 10000;
         entries = e2;
      }
   }

   if ( e2 != NULL )
      stack_free( e2 );

   return p;
}

struct string *merge( char *first, char *second )
{
   regmatch_t matches[ 1 ];
   int result, length;
   struct string *merged;
   char *ptr;

   result = regexec( &merge_regex, first, 1, matches, 0 );

   if ( result && result != REG_NOMATCH )
   {
      char err[ 80 ];

      regerror( result, &merge_regex, err, sizeof( err ));
      fprintf( stderr, "merge: regexec(): %s\n.", err );
      return NULL;
   }

   merged = make_string( );

   if ( result != REG_NOMATCH )
   {
      int i;

      length = strlen( first ) - ( matches[ 0 ].rm_eo - matches[ 0 ].rm_so );

      for( i = 0; i < length; ++i )
         string_append( merged, first[ i ] );
   }
   else
      string_assign( merged, first, strlen( first ));

   ptr = second;
   while( *ptr )
      string_append( merged, *ptr++ );

   return merged;
}

void *complete( char *name, int display )
{
   struct stack *p, *f;
   struct string *s, *completion = NULL;
   int i;
   char *source;

   p = f = NULL;
   source = expand_tilde( name );

AGAIN:
   p = find_possibilities( source );

   if ( p == NULL )
   {
      if ( display )
         return str_dup( "", 0 );

      completion = make_string();
   }
   else if ( p->used == 1 )
   {
      s = ( struct string *)p->values[ 0 ];

      completion = merge( source, s->str );

      if ( completion->str[ completion->used - 1 ] == '/' )
      {
         stack_free( p );
         string_free( s );
         free( source );
         source = str_dup( completion->str, completion->used );
         string_free( completion );

         goto AGAIN;
      }
   }
   else if ( p->used )
   {
      s = find_common_prefix( p );

      if ( s->used )
      {
         completion = merge( source, s->str );
         string_free( s );

         if ( completion->str[ completion->used - 1 ] == '/' )
         {
            for( i = 0; i < p->used; ++i )
               string_free( ( struct string *)p->values[ i ] );

            stack_free( p );
            free( source );
            source = str_dup( completion->str, completion->used );
            string_free( completion );

            goto AGAIN;
         }
      }
      else 
      {
         completion = make_string();
         string_assign( completion, source, strlen( source ));

         if ( completion->used && 
              completion->str[ completion->used - 1 ] != '/' )
            string_append( completion, '/' );
      }

      if ( display )
      {
         f = format_possibilities( p );

         while( f->used )
         {
            s = ( struct string *)stack_pop( f );
            write( STDOUT_FILENO, s->str, s->used );
            write( STDOUT_FILENO, "\n", 1 );
            string_free( s );
         }

         stack_free( f );
      }
      else
         f = format_possibilities( p );
   }

   if ( p != NULL && p->used )
      for( i = 0; i < p->used; ++i )
      {
         s = ( struct string *)p->values[ i ];
         string_free( s );
      }

   if ( p != NULL )
      stack_free( p );

   if ( completion == NULL )
   {
      completion = make_string();
      string_assign( completion, source, strlen( source ));
   }

   free( source );

   if ( !display )
   {
      if ( f == NULL )
         f = make_stack();

      stack_push( f, completion );
      return f;
   }

   name = str_dup( completion->str, completion->used );
   string_free( completion );

   return name;
}

int check_args( char *syntax, struct object *args )
{
   struct object *ptr, *item;
   int total, idx, count, type, result, limit, t, l;

   total = ( int)stack_pop( arg_stack );

   if ( total == 0 )
   {
      if ( args == NULL )
         return 0;

      print_err( ERR_ARGS, syntax, 0, -1 );
      return 1;
   }
   else if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      stack_truncate( arg_stack, total );
      return 1;
   }

   idx = arg_stack->used - total;
   limit = arg_stack->used;
   ptr = args;
   count = 0;

   while( ptr != NULL )
   {
      ++count;

      if ( idx == limit )
         break;

      type = ( int)arg_stack->values[ idx++ ];

      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, count, -1 );

         stack_truncate( arg_stack, total );
         return 1;
      }

      item = *( struct object **)stack->top;

      l = islist( item->flags );
      t = ( l ? -1 : type( item->data.atom->flags ));

      switch( type )
      {
         case ERR_SYMBOL:
            if ( l || t != ATOM_SYMBOL )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_SYMBOL );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_STRING:
            if ( l || t != ATOM_STRING )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_STRING );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_NUMBER:
            if ( l || t != ATOM_NUMBER )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_NUMBER );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_REGEXP:
            if ( l || t != ATOM_REGEXP )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_REGEXP );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_TABLE:
            if ( l || t != ATOM_TABLE )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_TABLE );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_STACK:
            if ( l || t != ATOM_STACK )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_STACK );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_ATOM:
            if ( l )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_ATOM );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_LIST:
            if ( l == 0 )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_LIST );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_CLOSURE:
            if ( l || ( t != ATOM_CLOSURE && t != ATOM_MACRO ))
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_CLOSURE );
               stack_truncate( arg_stack, total );
               return 1;
            }
            break;

         case ERR_CONSTANT:
            if ( l || t != ATOM_SYMBOL )
            {
               print_err( ERR_ARG_TYPE, syntax, count, ERR_SYMBOL );
               stack_truncate( arg_stack, total );
               return 1;
            }
            else if ( item->data.atom->data.value == ( double *)1 )
            {
               print_err( ERR_CONSTANT, syntax, count, -1 );
               stack_truncate( arg_stack, total );
               return 1;
            }

            break;
      }

      ptr = ptr->next;
   }

   if ( ptr != NULL )
   {
      print_err( ERR_MORE_ARGS, syntax, --count, -1 );
      result = 1;
   }
   else if ( idx < limit )
   {
      print_err( ERR_MISSING_ARG, syntax, ++count, -1 );
      result = 1;
   }
   else
      result = 0;

   stack_truncate( arg_stack, total );

   return result;
}

struct object *make_atom_from_number( double i )
{
   struct tree *entry;
   struct object *object;
   char buffer[ 128 ];

   snprintf( buffer, sizeof( buffer ), "%.10g", i );
   entry = get_id( buffer, strlen( buffer ));

   if ( entry->flags == 0 )
   {
      entry->flags = ATOM_NUMBER;
      entry->data.value = memory( sizeof( double ));
      *entry->data.value = i;
   }

   object = make_object();
   object->data.atom = entry;

   return object;
}

#ifdef SQL
struct object *make_atom_from_db( sqlite3 *db )
{
   struct tree *entry;
   struct object *obj;
   char buffer[ 64 ];

   snprintf( buffer, sizeof( buffer ), "<db#%d>", db_counter++ );

   entry = get_id( buffer, strlen( buffer ));
   entry->flags = ATOM_DB;
   entry->data.db = db;

   obj = make_object();
   obj->data.atom = entry;

   return obj;
}

int do_dbp( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( islist( car->flags ) == 0 && type( car->data.atom->flags ) == ATOM_DB )
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}
#endif

struct object *make_atom_from_string( char *s, int len )
{
   struct tree *entry;
   struct object *object;
   struct string *new;

   new = make_string();

   if ( len == -1 )
      len = strlen( s );

   string_assign( new, s, len );
   string_prepend( new, '"' );
   string_append( new, '"' );

   entry = get_id( new->str, new->used );
   string_free( new );

   if ( entry->flags == 0 )
   {
      entry->flags = ATOM_STRING;
      entry->data.string = memory( sizeof( struct lstring ));
      entry->data.string->length = len;
      entry->data.string->string = str_dup( s, entry->data.string->length );
   }

   object = make_object();
   object->data.atom = entry;

   return object;
}

struct object *make_atom_from_symbol( char *symbol )
{
   struct tree *entry;
   struct object *object;

   entry = get_id( symbol, strlen( symbol ));

   if ( entry->flags == 0 )
   {
      entry->flags = ATOM_SYMBOL;
      entry->data.value = NULL;
   }

   object = make_object();
   object->data.atom = entry;

   return object;
}

struct object *make_atom_from_regexp( regex_t *rx )
{
   struct tree *entry;
   struct object *object;
   char buffer[ 128 ];

   snprintf( buffer, sizeof( buffer ), "<REGEX#%d>", rx_counter++ );
   entry = get_id( buffer, strlen( buffer ));

   entry->flags = ATOM_REGEXP;
   entry->data.regexp = rx;

   object = make_object();
   object->data.atom = entry;

   return object;
}

struct object *duplicate_object( struct object *ptr )
{
   struct object *ptr2, *ptr3, *top;

   if ( ptr == NULL )
      return NULL;

   if ( islist( ptr->flags ) == 0 )
   {
      ptr2 = make_object();
      *ptr2 = *ptr;

      return ptr2;
   }

   top = make_object();
   setlist( top->flags );
   top->data.head = top->next = NULL;

   ptr2 = NULL;
   ptr = ptr->data.head;

   while( ptr != NULL )
   {
      ptr3 = ptr2;

      if ( islist( ptr->flags ) == 0 )
      {
         ptr2 = make_object();
         *ptr2 = *ptr;
      }
      else
         ptr2 = duplicate_object( ptr );

      if ( top->data.head == NULL )
         top->data.head = ptr2;
      else
         ptr3->next = ptr2;

      ptr = ptr->next;
   }

   return top;
}

int do_lines( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct winsize winsize;

      if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &winsize ) < 0 )
      {
         fprintf( stderr, "%s: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      stack_push( stack, make_atom_from_number( winsize.ws_row ));
   }

   return 0;
}

int do_cols( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct winsize winsize;

      if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &winsize ) < 0 )
      {
         fprintf( stderr, "%s: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      stack_push( stack, make_atom_from_number( winsize.ws_col ));
   }

   return 0;
}

int do_progn( char *syntax, struct object *args )
{
   struct object *ptr, *result = NULL;
   int i;

   if ( args == NULL )
   {
      fprintf( stderr, "%s: missing body.\n", syntax );
      return 1;
   }

   for( i = 1, ptr = args; ptr != NULL; ptr = ptr->next, ++i )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            fprintf( stderr, "%s: error evaluating body expression %d.\n",
                     syntax, i );
         return 1;
      }

      result = stack_pop( stack );
   }

   stack_push( stack, result );

   return 0;
}

int do_cons( char *syntax, struct object *args )
{
   struct object *car1, *car2, *new, *new2;

   stack_push( arg_stack, ( void *)-1);
   stack_push( arg_stack, ( void *)ERR_LIST );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   new2 = duplicate_object( car1 );
   new2->next = car2->data.head;

   new = make_object();
   setlist( new->flags );
   new->data.head = new2;

   stack_push( stack, new );

   return 0;
}

int do_quote( char *syntax, struct object *args )
{
   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   stack_push( stack, args );

   return 0;
}

int do_car( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_LIST );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( car->data.head == NULL )
   {
      fprintf( stderr, "%s: argument is empty list.\n", syntax );
      return 1;
   }
   else
      stack_push( stack, car->data.head );

   return 0;
}

int do_cdr( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_LIST );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( car->data.head == NULL )
   {
      fprintf( stderr, "%s: argument is empty list.\n", syntax );
      return 1;
   }
   else
   {
      stack_push( stack, make_object() );
      setlist( ( *( struct object **)stack->top )->flags );
      ( *( struct object **)stack->top )->data.head = car->data.head->next;
   }

   return 0;   
}

int do_eq( char *syntax, struct object *args )
{
   struct object *car1, *car2;
   int i;

   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   i = 0;

   if ( islist( car1->flags ) == 0 && 
        islist( car2->flags ) == 0 )
   {
      i = ( car1->data.atom == car2->data.atom );
   }
   else if ( islist( car1->flags ) == 1 && 
             islist( car2->flags ) == 1 )
   {
      i = (( car1->data.head == NULL && car2->data.head == NULL ) || car1 == car2 );
   }

   stack_push( stack, make_atom_from_number( i ));

   return 0;
}

int do_atom( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   stack_push( stack, ( islist( car->flags ) ? make_atom_from_number( 0 ) : 
                                               make_atom_from_number( 1 )));

   return 0;
}

int do_append( char *syntax, struct object *args )
{
   struct object *car1, *car2, *ptr;

   stack_push( arg_stack, ( void *)ERR_LIST );
   stack_push( arg_stack, ( void *)ERR_LIST );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   car2 = duplicate_object( car2 );

   ptr = car1->data.head;

   if ( ptr == NULL )
   {
      stack_push( stack, car2 );
      return 0;
   }

   car1 = duplicate_object( car1 );
   ptr = car1->data.head;

   while( ptr->next != NULL )
      ptr = ptr->next;

   ptr->next = car2->data.head;

   stack_push( stack, car1 );

   return 0;
}

int do_set( char *syntax, struct object *args )
{
   struct object *car1, *car2;

   stack_push( arg_stack, ( void *)ERR_CONSTANT );
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   if ( set_local( car1->data.atom->id, car2 ))
      insert_binding( car1->data.atom->id, car2 );

   stack_push( stack, car2 );

   return 0;
}

int do_eval( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 1, -1 );
      return 1;
   }

   return 0;
}

int do_if( char *syntax, struct object *args )
{
   struct object *result;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 1, -1 );
      return 1;
   }

   if ( args->next == NULL )
   {
      fprintf( stderr, "%s: missing consquent argument.\n", syntax );
      return 1;
   }

   stack_push( stack, args );

   if ( evaluate() )
   {
      if ( !stop )
         fprintf( stderr, "%s: evaluation of test expression failed.\n", 
                  syntax );
      return 1;
   }

   result = stack_pop( stack );

   if (( islist( result->flags ) == 1 && result->data.head == NULL ) || 
         result->data.atom == zero->data.atom ||
         result->data.atom == empty->data.atom )
   {
      if ( args->next->next == NULL )
      {
         stack_push( stack, result );
         return 0;
      }

      if ( do_progn( syntax, args->next->next ) )
      {
         if ( !stop )
            fprintf( stderr, 
                     "%s: evaluation of alternative expression(s) failed.\n",
                     syntax );
         return 1;
      }

      return 0;
   }

   stack_push( stack, args->next );

   if ( evaluate() )
   {
      if ( !stop )
         fprintf( stderr, "%s: evaluation of consequent expression failed.\n",
                  syntax );
      return 1;
   }

   return 0;
}

int do_and( char *syntax, struct object *args )
{
   struct object *ptr, *result = NULL;
   int i;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   i = 1;

   for( ptr = args; ptr != NULL; ptr = ptr->next )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, 0 );
         return 1;
      }

      result = stack_pop( stack );

      if (( islist( result->flags ) == 1 && result->data.head == NULL ) || 
            result->data.atom == zero->data.atom ||
            result->data.atom == empty->data.atom )
          break;

      ++i;
   }     

   stack_push( stack, result );

   return 0;
}

int do_or( char *syntax, struct object *args )
{
   struct object *ptr, *result = NULL;
   int i;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   for( i = 1, ptr = args; ptr != NULL; ptr = ptr->next, ++i )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 1;
      }

      result = stack_pop( stack );

      if (( islist( result->flags ) == 1 && result->data.head == NULL ) ||
            result->data.atom == zero->data.atom || 
            result->data.atom == empty->data.atom )
         continue;

      break;
   }

   stack_push( stack, result );

   return 0;
}

int do_list( char *syntax, struct object *args )
{
   struct object *ptr, **new, *result;
   int i, j;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   for( i = 1, ptr = args; ptr != NULL; ptr = ptr->next, ++i )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 1;
      }
   }

   result = make_object();
   setlist( result->flags );
   new = &result->data.head;

   --i;

   for( j = i; j; --j )
   {
      *new = duplicate_object( ( struct object *)stack->values[ stack->used - j ] );
      ( *new )->next = NULL;
      new = &( *new )->next;
   }

   stack_truncate( stack, i );
   stack_push( stack, result );

   return 0;
}

int do_not( char *syntax, struct object *args )
{
   struct object *result;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   if ( args->next )
   {
      print_err( ERR_MORE_ARGS, syntax, 1, -1 );
      return 1;
   }

   stack_push( stack, args );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 1, -1 );
      return 1;
   }

   result = stack_pop( stack );

   if (( islist( result->flags ) == 1 && result->data.head == NULL ) || 
         result->data.atom == zero->data.atom ||
         result->data.atom == empty->data.atom )
      stack_push( stack, make_atom_from_number( 1 ));
   else
      stack_push( stack, zero );

   return 0;
}

void do_print_objects_strings_unquoted( struct object *ptr, int recursive, int descr )
{
   while( ptr != NULL )
   {
      if ( islist( ptr->flags ) == 1 )
      {
         write( ( descr ? STDERR_FILENO : STDOUT_FILENO ), "(", 1 );
         do_print_objects_strings_unquoted( ptr->data.head, 1, descr );
         write( ( descr ? STDERR_FILENO : STDOUT_FILENO ), ")", 1 );
      }
      else
      {
         char *str;
         int len;

         if ( recursive == 0 && type( ptr->data.atom->flags ) == ATOM_STRING )
         {
            str = ptr->data.atom->data.string->string;
            len = ptr->data.atom->data.string->length;
         }
         else
         {
            str = ptr->data.atom->syntax;
            len = ptr->data.atom->len;
         }

         write( ( descr ? STDERR_FILENO : STDOUT_FILENO ), str, len );
      }

      if ( recursive == 0 )
         break;

      if (( ptr = ptr->next ) != NULL )
         write( ( descr ? STDERR_FILENO : STDOUT_FILENO ), " ", 1 );
   }
}

int do_print( char *syntax, struct object *args )
{
   struct object *ptr;
   int i;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   for( i = 1, ptr = args; ptr != NULL; ptr = ptr->next, ++i )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 1;
      }

      do_print_objects_strings_unquoted( stack_pop( stack ), 0, 0 );
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_newline( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   write( STDOUT_FILENO, "\n", 1 );
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_load( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   return load( car->data.atom->data.string->string );
}

void nocanon( char *syntax )
{
   if ( mode == 0 )
      return;

   if ( isatty( STDIN_FILENO ) )
   {
      struct termios termios;
      
      termios = canon_termios;

      termios.c_lflag &= ~( ECHO | ICANON | IEXTEN | ISIG );
      termios.c_iflag &= ~( IXON | IXOFF );
      termios.c_cc[ VMIN ] = 1;
      termios.c_cc[ VTIME ] = 0;
 
      if ( tcsetattr( STDIN_FILENO, TCSAFLUSH, &termios ) < 0 )
      {
         fprintf( stderr, "%s: tcsetattr: %s.\n", syntax, strerror( errno ));
         return;
      }

      mode = 0;
   }

   return;
}

void canon( char *syntax )
{
   if ( mode )
      return;

   if ( isatty( STDIN_FILENO ) )
   {
      if ( tcsetattr( STDIN_FILENO, TCSAFLUSH, &canon_termios ) < 0 )
      {
         fprintf( stderr, "%s: tcsetattr: %s.\n", syntax, strerror( errno ));
         return;
      }

      mode = 1;
   }

   return;
}

int do_complete( char *syntax, struct object *args )
{
   struct object *car, **ptr, *result;
   struct stack *results;
   struct string *str;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   results = ( struct stack *)complete( car->data.atom->data.string->string, 0 );

   result = make_object();
   setlist( result->flags );
   ptr = &result->data.head;

   while( results->used )
   {
      str = ( struct string *)stack_pop( results );
      *ptr = make_atom_from_string( str->str, str->used );
      string_free( str );

      ptr = &( *ptr )->next;
   }

   stack_free( results );
   stack_push( stack, result );

   return 0;
}

int getline_from_file( char *syntax, int reset )
{
   static char buffer[ 102400 ] = "", *ptr = buffer;
   int result;
   static int len = 0;
   struct string *s;

   switch( reset )
   {
      case 1:
         stack_push( input_buffer_stack, str_dup( ptr, len ));
         stack_push( input_buffer_stack, ( void *)len );

         buffer[ 0 ] = '\0';
         len = 0;
         ptr = buffer;
         return 0;

      case 2:
         if ( input_buffer_stack->used )
         {
            len = ( int )stack_pop( input_buffer_stack );
            ptr = ( char * )stack_pop( input_buffer_stack );
            bcopy( ptr, buffer, len );
            free( ptr );
            ptr = buffer;
         }
         else
         {
            buffer[ 0 ] = '\0';
            ptr = buffer;
            len = 0;
         }

         return 0;
   }

   s = make_string();

   for( ; ; )
   {
      if ( len == 0 )
      {
         if (( result = read( STDIN_FILENO, buffer, sizeof( buffer ) - 1 )) < 0 )
         {
            if ( errno == EINTR )
               continue;

            fprintf( stderr, "%s: read: %s.\n", syntax, strerror( errno ));
            string_free( s );
            return 1;
         }
         else if ( result == 0 )
         {
            len = 0;

            if ( s->used )
               stack_push( stack, make_atom_from_string( s->str, s->used ));
            else
               stack_push( stack, make_atom_from_number( 0 ));

            string_free( s );
            return 0;
         }

         buffer[ result ] = '\0';
         ptr = buffer;
         len = result;
      }

      while( len )
      {
         string_append( s, *ptr );
         --len;

         if ( *ptr++ == 10 )
            goto NEXT;
      }
   }

NEXT:
   stack_push( stack, make_atom_from_string( s->str, s->used ));
   string_free( s );

   return 0;
}

int do_getline( char *syntax, struct object *args )
{
   struct object *car1 = NULL, *car2 = NULL;

   if ( args != NULL )
   {
      if ( args->next != NULL && args->next->next != NULL )
      {
         print_err( ERR_MORE_ARGS, syntax, 2, -1 );
         return 1;
      }

      stack_push( stack, args );

      if ( evaluate())
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, 1, -1 );
         return 1;
      }

      car1 = stack_pop( stack );

      if ( islist( car1->flags ) == 1 || 
          type( car1->data.atom->flags ) != ATOM_STRING )
      {
         print_err( ERR_ARG_TYPE, syntax, 1, ERR_STRING );
         return 1;
      }

      if ( args->next != NULL )
      {
         stack_push( stack, args->next );

         if ( evaluate() )
         {
            if ( !stop )
               print_err( ERR_EVAL, syntax, 2, -1 );
            return 1;
         }

         car2 = stack_pop( stack );

         if ( islist( car2->flags ) == 1 || 
               type( car2->data.atom->flags ) != ATOM_NUMBER )
         {
            print_err( ERR_ARG_TYPE, syntax, 2, ERR_NUMBER );
            return 1;
         }
      }
   }

   if ( isatty( STDIN_FILENO ) == 0 )
      return getline_from_file( syntax, 0 );

   {
      struct string *s;
      struct stack *offsets;
      char c, *name;
      int tabstop, result, offset, eof, was_canon, name_len, fd;

      if ( car2 == NULL )
         tabstop = 8;
      else
         tabstop = *car2->data.atom->data.value;

      eof = 0;
      fd = STDOUT_FILENO;

      if ( isatty( STDOUT_FILENO ) == 0 )
      {
         if ( isatty( STDERR_FILENO ))
            fd = STDERR_FILENO;
         else
            fd = -1;
      }

      was_canon = mode;
      nocanon( syntax );

      if ( car1 != NULL )
      {
         name = car1->data.atom->data.string->string;
         name_len = car1->data.atom->data.string->length;

         if ( fd > 0 )
            write( fd, name, name_len );
      }
      else
      {
         name = "";
         name_len = 0;
      }

      offset = 0;
      s = make_string();
      offsets = make_stack();

      for( ; ; )
      {
         result = read( STDIN_FILENO, &c, 1 );

         if ( result < 0 )
         {
            if ( errno == EINTR )
               continue;

            if ( was_canon )
               canon( syntax );

            string_free( s );
            stack_free( offsets );

            fprintf( stderr, "%s: read: %s.\n", syntax, strerror( errno ));
            return 1;
         }
         else if ( result == 0 )
         {
            eof = 1;
            break;
         }

         switch( c )
         {
            case '\010':
               if ( s->used )
               {
                  if ( fd > 0 )
                  {
                     if ( s->str[ s->used - 1 ] == '\t' )
                     {
                        result = ( int)stack_pop( offsets );
                        offset -= result - 1;
                     }
                     else
                        result = 1;

                     while( result-- )
                        write( fd, "\010 \010", 3 );
                  }

                  string_chop( s );
               }
               break;

            case '\025':
               if ( fd > 0 )
               {
                  result = s->used + offset;
                  while( result-- )
                     write( fd, "\010 \010", 3 );

                  stack_truncate( offsets, offsets->used );
               }

               string_truncate( s );
               offset = 0;
               break;

            case '\027':
               while( s->used && !isalnum( s->str[ s->used - 1 ] ))
               {
                  if ( fd > 0 )
                  {
                     if ( s->str[ s->used - 1 ] == '\t' )
                     {
                        result = ( int )stack_pop( offsets );
                        offset -= result - 1;
                     }
                     else
                        result = 1;

                     while( result-- )
                        write( fd, "\010 \010", 3 );
                  }

                  string_chop( s );
               }

               while( s->used && isalnum( s->str[ s->used - 1 ] ))
               {
                  if ( fd > 0 ) 
                     write( fd, "\010 \010", 3 );

                  string_chop( s );
               }

               break;

            case '\011':
               if ( tabstop )
               {
                  if ( fd > 0 )
                  {
                     result = tabstop - ( s->used + offset ) % tabstop;
                     offset += result - 1;
                     stack_push( offsets, ( void *)result );

                     while( result-- )
                        write( fd, " ", 1 );
                  }

                  string_append( s, c );
               }
               else
               {
                  char *ptr, *ptr2;
                  int length;

                  if ( s->used )
                  {
                     ptr = &s->str[ s->used - 1 ]; 
                     while( ptr > s->str && !isspace( *ptr ))
                        --ptr;

                     if ( isspace( *ptr ))
                        ++ptr;
                  }
                  else
                     ptr = s->str;

                  if ( fd > 0 )
                     write( fd, "\n", 1 );

                  length = strlen( ptr );
                  ptr2 = ptr = ( char *)complete( ptr, 1 );

                  length = s->used - length;
                  while( s->used > length )
                     string_chop( s );

                  while( *ptr )
                     string_append( s, *ptr++ );

                  if ( fd > 0 )
                  {
                     write( fd, name, name_len );
                     write( fd, s->str, s->used );
                  }

                  free( ptr2 );
               }

               break;

            case '\r':
            case '\n':
               string_append( s, '\n' );
               if ( fd > 0 )
                  write( fd, "\r\n", 2 );
               goto BREAK;

            case '\004':
               eof = 1;
               goto BREAK;

            default:
               if ( c > 31 )
               {
                  string_append( s, c );
                  if ( fd > 0 )
                     write( fd, &c, 1 );
               }
               break;

         } /* end of switch */

      } /* end of for loop */

   BREAK:
      if ( was_canon )
         canon( syntax );

      stack_free( offsets );

      if ( !s->used && eof )
         stack_push( stack, make_atom_from_number( 0 ));
      else
         stack_push( stack, make_atom_from_string( s->str, s->used ));

      string_free( s );
   }

   return 0;
}

int do_numberp( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( islist( car->flags ))
   {
      stack_push( stack, make_atom_from_number( 0 ));
      return 0;
   }

   if ( type( car->data.atom->flags ) == ATOM_NUMBER )
      stack_push( stack, make_atom_from_number( 1 ));
   else
      stack_push( stack, make_atom_from_number( 0 ));

   return 0;
}

int do_stringp( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( islist( car->flags ))
   {
      stack_push( stack, make_atom_from_number( 0 ));
      return 0;
   }

   if ( type( car->data.atom->flags ) == ATOM_STRING )
      stack_push( stack, make_atom_from_number( 1 ));
   else
      stack_push( stack, make_atom_from_number( 0 ));

   return 0;
}

int do_split( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   car1 = args;
   car2 = args->next;

   if ( car2 == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, -1 );
      return 1;
   }

   car3 = car2->next;

   if ( car3 != NULL )
   {
      if ( car3->next )
      {
         print_err( ERR_MORE_ARGS, syntax, 3, -1 );
         return 1;
      }
   }

   stack_push( stack, car1 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 1, -1 );
      return 1;
   }

   stack_push( stack, car2 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 2, -1 );
      return 1;
   }

   if ( car3 != NULL )
   {
      stack_push( stack, car3 );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, 3, -1 );
         return 1;
      }

      car3 = stack_pop( stack );

      if ( islist( car3->flags ) == 1 || 
          type( car3->data.atom->flags ) != ATOM_NUMBER )
      {
         print_err( ERR_ARG_TYPE, syntax, 3, ERR_NUMBER );
         return 1;
      }

      if ( *car3->data.atom->data.value <= 0 )
      {
         fprintf( stderr, "%s: argument 3 <= 0.\n", syntax );
         return 1;
      }
   }

   car2 = stack_pop( stack );

   if ( islist( car2->flags ) == 1 || 
       type( car2->data.atom->flags ) != ATOM_STRING )
   {
      print_err( ERR_ARG_TYPE, syntax, 2, ERR_STRING );
      return 1;
   }

   car1 = stack_pop( stack );

   if ( islist( car1->flags ) == 1 || 
       type( car1->data.atom->flags )!= ATOM_STRING )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, ERR_STRING );
      return 1;
   }

   {
      struct object *result;
      char *tmp1, *tmp2, *tmp3, *tmp4 = NULL, **results;
      int i, len, length, arg3, duped = 0;

      results = NULL;

      if ( car2->data.atom->data.string->length == 0 )
      {
         result = make_object();
         setlist( result->flags );
         result->data.head = car2;
         stack_push( stack, result );
         return 0;
      }


      tmp1 = car1->data.atom->data.string->string;
      tmp2 = car2->data.atom->data.string->string;
      length = car2->data.atom->data.string->length;
      len = length;

      arg3 = -1;

      if ( car3 != NULL )
      {
         if ( *car3->data.atom->data.value <= length )
            arg3 = ( *car3->data.atom->data.value ) - 1;
      }

      if ( arg3 == 0 )
      {
         result = make_object();
         setlist( result->flags );
         result->data.head = 
            make_atom_from_string( car2->data.atom->data.string->string,
                                   car2->data.atom->data.string->length );
         stack_push( stack, result );
         return 0;
      }
      else if ( *tmp1 == '\0' )
      {
         results = ( char **)memory( sizeof( char * ) * ( length + 1 ));

         for( i = 0; i <= length; ++i )
            results[ i ] = NULL;

         if ( arg3 > 0 && arg3 < length )
            length = arg3;

         for( i = 0; i < length; ++i )
         {
            results[ i ] = ( char *)memory( 2 );
            results[ i ][ 0 ] = tmp2[ i ];
            results[ i ][ 1 ] = '\0';
         }

         if ( arg3 > 0 && arg3 == length )
            results[ i ] = str_dup( &tmp2[ i ], len - i );
      }
      else
      {
         char *start;

         results = ( char **)memory( sizeof( char * ) * ( length + 1 ));

         for( i = 0; i <= length; ++i )
            results[ i ] = NULL;

         i = 0;
         tmp4 = tmp3 = str_dup( tmp2, strlen( tmp2 ));
         duped = 1;

         if ( arg3 > 0 && arg3 < length )
            length = arg3;

         while( i < length && ( start = strsep( &tmp3, tmp1 )) != NULL )
            results[ i++ ] = start;

         if ( arg3 > 0 && arg3 == length && tmp3 != NULL )
            results[ i ] = tmp3;
      }

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      if ( *results == NULL )
         result->data.head = car2;
      else
      {
         char **ptr;
         struct object **ptr2;

         ptr2 = &result->data.head;

         for( ptr = results; *ptr != NULL; ++ptr )
         {
            *ptr2 = make_atom_from_string( *ptr, strlen( *ptr ));
            ptr2 = &( *ptr2 )->next;
            if ( duped == 0 )
               free( *ptr );
         }
      }

      free( results );

      if ( duped )
         free( tmp4 );
   }

   return 0;
}

int get_join_args( struct object *args, int i, char *syntax )
{
   struct object *ptr;

   for( ptr = args; ptr != NULL; ptr = ptr->next, ++i )
   {
      if ( islist( ptr->flags ))
      {
         if ( ptr->data.head == NULL )
         {
            print_err( ERR_ARG_TYPE, syntax, i, ERR_STRING );
            return 0;
         }

         if (( i = get_join_args( ptr->data.head, i, syntax )) == 0 )
            return 0;

         --i;
      }
      else
      {
         if ( type( ptr->data.atom->flags ) != ATOM_STRING )
         {
            print_err( ERR_ARG_TYPE, syntax, i, ERR_STRING );
            return 0;
         }

         stack_push( stack, ptr );
      }
   }

   return i;
}

int process_join_args( struct object *args, int i, char *syntax )
{
   struct object *ptr, *result;

   for( ptr = args; ptr != NULL; ptr = ptr->next, ++i )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 0;
      }

      result = *( struct object **)stack->top;

      if ( islist( result->flags ))
      {
         int j;

         if ( result->data.head == NULL )
         {
            print_err( ERR_ARG_TYPE, syntax, i, ERR_STRING );
            return 0;
         }

         j = stack->used;

         if (( i = get_join_args( result->data.head, i, syntax )) == 0 )
            return 0;

         --i;

         for( ; j < stack->used; ++j )
            stack->values[ j - 1 ] = stack->values[ j ];

         stack_pop( stack );
      }
      else if ( type( result->data.atom->flags ) != ATOM_STRING )
      {
         print_err( ERR_ARG_TYPE, syntax, i, ERR_STRING );
         return 0;
      }
   }

   return i;
}

int do_join( char *syntax, struct object *args )
{
   struct object *car1, *car2;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   car1 = args;
   car2 = car1->next;

   if ( car2 == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, -1 );
      return 1;
   }

   stack_push( stack, car1 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 1, -1 );
      return 1;
   }

   {
      int i, j, len;
      char *tmp;
      struct string *buffer;

      if (( i = process_join_args( car2, 1, syntax )) == 0 )
         return 1;

      car1 = stack->values[ stack->used - i ];

      if ( islist( car1->flags ) == 1 || 
           type( car1->data.atom->flags ) != ATOM_STRING )
      {
         print_err( ERR_ARG_TYPE, syntax, 1, ERR_STRING );
         return 1;
      }

      buffer = make_string();

      for( j = i - 1; j; --j )
      {
         struct object *item;

         item = stack->values[ stack->used - j ];
         tmp = item->data.atom->data.string->string;
         len = item->data.atom->data.string->length;

         while( len-- )
            string_append( buffer, *tmp++ );

         if ( j != 1 )
         {
            tmp = car1->data.atom->data.string->string;
            len = car1->data.atom->data.string->length;

            while( len-- )
               string_append( buffer, *tmp++ );
         }
      }

      stack_truncate( stack, i );
      stack_push( stack, make_atom_from_string( buffer->str, buffer->used ));
      string_free( buffer );
   }

   return 0;
}

int apply_regexp( regex_t *regexp, char *the_string, int len, int show_offset )
{
   struct object **ptr = NULL;
   struct string *buffer;
   regmatch_t matches[ 20 ];
   int result, length, i;

   matches[ 0 ].rm_so = 0;
   matches[ 0 ].rm_eo = len;

   result = regexec( regexp, the_string, 
                     ( show_offset == 2 ? 1 : 20 ), matches, REG_STARTEND );

   stack_push( stack, make_object());
   setlist( ( *( struct object **)stack->top )->flags );

   if ( result )
   {
      char err[ 80 ];

      if ( result == REG_NOMATCH )
         return 0;

      regerror( result, regexp, err, sizeof( err ));
      fprintf( stderr, "apply_regexp: regexec: %s.\n", err );
      return 1;
   }

   switch ( show_offset )
   {  
      case 0:
         ptr = &( *( struct object **)stack->top )->data.head;
         break;

      case 1:
         ( *( struct object **)stack->top )->data.head = 
            make_atom_from_number( matches[ 0 ].rm_so );
         ( *( struct object **)stack->top )->data.head->next = 
            make_atom_from_number( matches[ 0 ].rm_eo );
         ptr = &( *( struct object **)stack->top )->data.head->next->next;
         break;

      case 2:
         ( *( struct object **)stack->top )->data.head = 
            make_atom_from_number( matches[ 0 ].rm_so );
         ( *( struct object **)stack->top )->data.head->next = 
            make_atom_from_number( matches[ 0 ].rm_eo );
         return 0;
   }

   buffer = make_string();

   for( i = 0; i < 20; ++i )
   {
      if ( matches[ i ].rm_so >= 0 )
      {
         int j;

         length = matches[ i ].rm_so + matches[ i ].rm_eo - matches[ i ].rm_so;

         for( j = matches[ i ].rm_so; j < length; ++j )
            string_append( buffer, the_string[ j ] );
      }

      *ptr = make_atom_from_string( buffer->str, buffer->used );
      ( *ptr )->next = NULL;
      ptr = &( *ptr )->next;

      string_truncate( buffer );
   }

   string_free( buffer );

   return 0;
}

int do_matches( char *syntax, struct object *args )
{
   struct object *car1, *car2;

   stack_push( arg_stack, ( void *)ERR_REGEXP );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   return apply_regexp( car1->data.atom->data.regexp, 
                        car2->data.atom->data.string->string, 
                        car2->data.atom->data.string->length,
                        0 );
}

int do_match( char *syntax, struct object *args )
{
   struct object *car1, *car2;

   stack_push( arg_stack, ( void *)ERR_REGEXP );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   return apply_regexp( car1->data.atom->data.regexp, 
                        car2->data.atom->data.string->string, 
                        car2->data.atom->data.string->length,
                        2 );
}

int add_char( struct string *buffer, char *ptr, int change_case )
{
   switch( change_case )
   {
      case 0:
         string_append( buffer, *ptr );
         break;

      case 1:
         string_append( buffer, toupper( *ptr ));
         change_case = 0;
         break;

      case 2:
         string_append( buffer, toupper( *ptr ));
         break;

      case 3:
         string_append( buffer, tolower( *ptr ));
         change_case = 0;
         break;

      case 4:
         string_append( buffer, tolower( *ptr ));
         break;
   }

   return change_case;
}

int do_substitute( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3, *car4;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   car1 = args;
   car2 = car1->next;

   if ( car2 == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, -1 );
      return 1;
   }

   car3 = car2->next;

   if ( car3 == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 3, -1 );
      return 1;
   }

   stack_push( stack, car1 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 1, -1 );
      return 1;
   }

   stack_push( stack, car2 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 2, -1 );
      return 1;
   }

   stack_push( stack, car3 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 3, -1 );
      return 1;
   }

   car4 = car3->next;

   if ( car4 != NULL )
   {
      if ( car4->next != NULL )
      {
         print_err( ERR_MORE_ARGS, syntax, 4, -1 );
         return 1;
      }

      stack_push( stack, car4 );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, 4, -1 );
         return 1;
      }

      car4 = stack_pop( stack );

      if ( islist( car4->flags ) == 1 || 
          type( car4->data.atom->flags ) != ATOM_NUMBER )
      {
         print_err( ERR_ARG_TYPE, syntax, 4, ERR_NUMBER );
         return 1;
      }
   }

   car3 = stack_pop( stack );

   if ( islist( car3->flags ) == 1 || 
        type( car3->data.atom->flags ) != ATOM_STRING )
   {
      print_err( ERR_ARG_TYPE, syntax, 3, ERR_STRING );
      return 1;
   }

   car2 = stack_pop( stack );

   if ( islist( car2->flags ) == 1 || 
       type( car2->data.atom->flags ) != ATOM_STRING )
   {
      print_err( ERR_ARG_TYPE, syntax, 2, ERR_STRING );
      return 1;
   }

   car1 = stack_pop( stack );

   if ( islist( car1->flags ) == 1 || 
       type( car1->data.atom->flags ) != ATOM_REGEXP )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, ERR_REGEXP );
      return 1;
   }

   {
      char *tmp1, *tmp2, *tmp3, *tmp4, *subs[ 11 ], *ptr, *old_tmp4;
      struct object *obj;
      struct string *buffer;
      int first, arg4, len3, escape, i, begin, end, count, change_case;
      regex_t *rx;

      rx = car1->data.atom->data.regexp;

      arg4 = ( car4 == NULL ? 1 : *car4->data.atom->data.value );

      tmp2 = car2->data.atom->data.string->string;
      tmp3 = car3->data.atom->data.string->string;
      len3 = car3->data.atom->data.string->length;
      tmp4 = tmp3;

      change_case = 0;

      buffer = make_string();
      count = 0;
      old_tmp4 = NULL;

      first = 1;

      for( ; ; )
      {
         if ( !first && ( tmp4 - tmp3 ) >= len3 )
            break;

         first = 0;

         if ( tmp4 == old_tmp4 )
            string_append( buffer, *tmp4++ );

         old_tmp4 = tmp4;

         if ( apply_regexp( rx, tmp4, len3 - ( tmp4 - tmp3 ), 1 ))
            return 1;

         car1 = stack_pop( stack );

         if ( car1->data.head == NULL )
            break;

         begin = *car1->data.head->data.atom->data.value;
         end = *car1->data.head->next->data.atom->data.value;

         car1->data.head->flags = 0;
         car1->data.head->next->flags = 0;

         i = 0;

         for( obj = car1->data.head->next->next; obj != NULL; obj = obj->next )
         {
            subs[ i++ ] = obj->data.atom->data.string->string;

            if ( i > 10 )
               break;
         }

         ptr = tmp4;

         if ( begin )
            for( i = 0; i < begin; ++i )
               string_append( buffer, *ptr++ );

         escape = 0;

         for( ptr = tmp2; *ptr; ++ptr )
         {
            char c[ 2 ];

            if ( *ptr == '\\' )
            {
               if ( escape )
                  string_append( buffer, '\\' );

               escape ^= 1;
               continue;
            }
            else if ( escape )
            {
               c[ 0 ] = *ptr;
               c[ 1 ] = '\0';

               escape = 0;

               if ( *ptr >= '1' && *ptr <= '9' )
               {
                  tmp1 = subs[ atoi( c ) ];
                  while( *tmp1 )
                  {
                     change_case = add_char( buffer, tmp1, change_case );
                     ++tmp1;
                  }
                  continue;
               }

               switch( *ptr )
               {
                  case '0':
                     tmp1 = subs[ 10 ];
                     while( *tmp1 )
                     {
                        change_case = add_char( buffer, tmp1, change_case );
                        ++tmp1;
                     }
                     continue;

                  case '&':
                     tmp1 = subs[ 0 ];
                     while( *tmp1 )
                     {
                        change_case = add_char( buffer, tmp1, change_case );
                        ++tmp1;
                     }
                     continue;

                  case 't':
                     string_append( buffer, '\t' );
                     continue;

                  case 'b':
                     string_append( buffer, ' ' );
                     continue;

                  case 'U':
                     change_case = 2;
                     continue;

                  case 'u':
                     change_case = 1;
                     continue;

                  case 'L':
                     change_case = 4;
                     continue;

                  case 'l':
                     change_case = 3;
                     continue;

                  case 'e':
                     change_case = 0;
                     continue;

                  default:
                     string_append( buffer, '\\' );
                     continue;
               }
            }

            change_case = add_char( buffer, ptr, change_case );
         }

         tmp4 = &tmp4[ end ];

         if ( ++count == arg4 )
            break;
      }

      for( ptr = tmp4; *ptr; ++ptr )
         change_case = add_char( buffer, ptr, change_case );

      stack_push( stack, make_atom_from_string( buffer->str, buffer->used ));
      string_free( buffer );
   }

   return 0;
}

int do_regcomp( char *syntax, struct object *args )
{
   struct object *car;
   struct string *new;
   char *ptr, *tmp;
   regex_t *regexp;
   int escape, result, len;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   ptr = car->data.atom->data.string->string;
   len = car->data.atom->data.string->length;

   escape = 0;

   new = make_string();

   for( ; len; --len )
   {
      if ( *ptr == '\\' )
      {
         escape ^= 1;

         if ( !escape )
         {
            string_append( new, '\\' );
            string_append( new, '\\' );
         }

         ++ptr;
         continue;
      }

      if ( escape )
      {
         switch( *ptr )
         {
            case 'b':
               string_append( new, ' ' );
               break;

            case 't':
               string_append( new, '\t' );
               break;

            case 'n':
               string_append( new, '\n' );
               break;

            case '>':
               tmp = "[[:>:]]";
               while( *tmp )
                  string_append( new, *tmp++ );
               break;

            case '<':
               tmp = "[[:<:]]";
               while( *tmp )
                  string_append( new, *tmp++ );
               break;

            case '?':
               string_append( new, '\\' );
               string_append( new, '?' );
               break;

            case '+':
               string_append( new, '\\' );
               string_append( new, '+' );
               break;

            case '^':
               string_append( new, '\\' );
               string_append( new, '^' );
               break;

            case '$':
               string_append( new, '\\' );
               string_append( new, '$' );
               break;

            case '.':
               string_append( new, '\\' );
               string_append( new, '.' );
               break;

            case '[':
               string_append( new, '\\' );
               string_append( new, '[' );
               break;

            case '(':
               string_append( new, '\\' );
               string_append( new, '(' );
               break;

            case ')':
               string_append( new, '\\' );
               string_append( new, ')' );
               break;

            case '|':
               string_append( new, '\\' );
               string_append( new, '|' );
               break;

            case '{':
               string_append( new, '\\' );
               string_append( new, '{' );
               break;

            case '*':
               string_append( new, '\\' );
               string_append( new, '*' );
               break;

            default:
               string_append( new, *ptr );
         }
      }
      else
         string_append( new, *ptr );

      ++ptr;
      escape = 0;
   }

   regexp = ( regex_t *)memory( sizeof( regex_t ));
   regexp->re_endp = &new->str[ new->used ];
   result = regcomp( regexp, new->str, REG_EXTENDED | REG_PEND );

   if ( result )
   {
      char err[ 83 ];

      regerror( result, regexp, err, sizeof( err ) - 1 );
      free( regexp );
      string_free( new );

      stack_push( stack, make_atom_from_string( err, strlen( err )));
      return 0;
   }

   string_free( new );

   stack_push( stack, make_atom_from_regexp( regexp ));

   return 0;
}

int do_sort( char *syntax, struct object *args )
{
   struct object *ptr, *result, **ptr2;
   struct stack *items;
   int i, first = 1, type = 0;

   for( i = 1, ptr = args; ptr != NULL; ptr = ptr->next, ++i )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 1;
      }

      result = *( struct object **)stack->top;

      if ( islist( result->flags ))
      {
         print_err( ERR_ARG_TYPE, syntax, i, ERR_ATOM );
         return 1;
      }

      if ( type( result->data.atom->flags ) != ATOM_STRING && 
           type( result->data.atom->flags ) != ATOM_NUMBER )
      {
         fprintf( stderr, "%s: arguments must either be all strings "
                  "or all numbers.\n", syntax );
         return 1;
      }

      if ( first && 
           type( result->data.atom->flags ) == ATOM_STRING )
         type = 1;

      first = 0;

      if ( type )
      {
         if ( type( result->data.atom->flags ) != ATOM_STRING )
         {
            print_err( ERR_ARG_TYPE, syntax, i, ERR_STRING );
            return 1;
         }
      }
      else
      {
         if ( type( result->data.atom->flags ) != ATOM_NUMBER )
         {
            print_err( ERR_ARG_TYPE, syntax, i, ERR_NUMBER );
            return 1;
         }
      }
   }

   items = make_stack();
   while( --i )
   {
      result = stack_pop( stack );
      stack_push( items, ( type ? ( struct object *)result->data.atom->data.string : 
                                  ( struct object *)result->data.atom->data.value ));
   }

   result = make_object();
   setlist( result->flags );
   stack_push( stack, result );

   ptr2 = &result->data.head;

   if ( type )
   {
      qsort( items->values, items->used, sizeof( void * ), 
             compare_strings );

      for( i = 0; i < items->used; ++i )
      {
         *ptr2 = make_atom_from_string( 
            (( struct lstring *)items->values[ i ] )->string,
            (( struct lstring *)items->values[ i ] )->length );
         ( *ptr2 )->next = NULL;
         ptr2 = &( *ptr2 )->next;
      }
   }
   else
   {
      qsort( items->values, items->used, sizeof( void * ), 
             compare_numbers );

      for( i = 0; i < items->used; ++i )
      {
         *ptr2 = make_atom_from_number( *( double *)items->values[ i ] );
         ( *ptr2 )->next = NULL;
         ptr2 = &( *ptr2 )->next;
      }
   }

   stack_free( items );

   return 0;
}

int do_sortcar( char *syntax, struct object *args )
{
   struct object *car, *ptr, **ptr2 = NULL;
   int type = 0, i, first = 1;

   stack_push( arg_stack, ( void *)ERR_LIST );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   i = 1;

   for( ptr = car->data.head; ptr != NULL; ptr = ptr->next )
   {
      if ( islist( ptr->flags ) == 0 )
      {
         fprintf( stderr, "%s: list element %d is not a sublist.\n", syntax, i );
         return 1;
      }

      if ( islist( ptr->data.head->flags ))
      {
         fprintf( stderr, "%s: car of sublist %d is not an atom.\n",
                  syntax, i );
         return 1;
      }

      if ( first )
      {
         if ( type( ptr->data.head->data.atom->flags ) == ATOM_STRING )
            type = 1;
         else if ( type( ptr->data.head->data.atom->flags ) == ATOM_NUMBER )
            type = 0;
         else
         {
            fprintf( stderr, "%s: the car of sublist %d is neither a number"
                     "nor a string.\n", syntax, i );
            return 1;
         }

         first = 0;
      }
      else if ( type )
      {
         if ( type( ptr->data.head->data.atom->flags ) != ATOM_STRING )
         {
            fprintf( stderr, "%s: the car of sublist %d is not a string.\n", 
                     syntax, i );
            return 1;
         }
      }
      else
      {
         if ( type ( ptr->data.head->data.atom->flags ) != ATOM_NUMBER )
         {
            fprintf( stderr, "%s: the car of sublist %d is not a number.\n", 
                     syntax, i ); 
            return 1;
         }
      }

      ++i;
   }

   {
      struct object *result;
      struct stack *items;

      items = make_stack();

      for( ptr = car->data.head; ptr != NULL; ptr = ptr->next )
         stack_push( items, ptr );

      result = make_object();
      stack_push( stack, result );
      setlist( result->flags );

      ptr2 = &result->data.head;

      if ( type )
         qsort( items->values, items->used, sizeof( struct object * ), 
                compare_car_strings );
      else
         qsort( items->values, items->used, sizeof( struct object * ), 
                compare_car_numbers );

      for( i = 0; i < items->used; ++i )
      {
         *ptr2 = duplicate_object( ( struct object *)items->values[ i ] );
         ptr2 = &( *ptr2 )->next;
      }

      *ptr2 = NULL;
      stack_free( items );
   }

   return 0;
}

int do_sortlist( char *syntax, struct object *args )
{
   struct object *car, *ptr, **ptr2 = NULL;
   int type = 0, i, first = 1;

   stack_push( arg_stack, ( void *)ERR_LIST );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   for( i = 1, ptr = car->data.head; ptr != NULL; ptr = ptr->next, ++i )
   {
      if ( islist( ptr->flags ) == 1 )
      {
         fprintf( stderr, "%s: list element %d is not an atom.\n", syntax, i );
         return 1;
      }

      if ( first )
      {
         if ( type( ptr->data.atom->flags ) == ATOM_STRING )
            type = 1;
         else if ( type( ptr->data.atom->flags ) == ATOM_NUMBER )
            type = 0;
         else
         {
            fprintf( stderr, "%s: list element %d is neither a number nor a "
                     "string.\n", syntax, i );
            return 1;
         }

         first = 0;

      }
      else if ( type )
      {
         if ( type( ptr->data.atom->flags ) != ATOM_STRING )
         {
            fprintf( stderr, "%s: list element %d is not a string.\n", 
                     syntax, i );
            return 1;
         }
      }
      else
      {
         if ( type( ptr->data.atom->flags ) != ATOM_NUMBER )
         {
            fprintf( stderr, "%s: list element %d is not a number.\n", 
                     syntax, i ); 
            return 1;
         }
      }
   }

   {
      struct object *result;
      struct stack *items;

      items = make_stack();

      for( ptr = car->data.head; ptr != NULL; ptr = ptr->next )
      {
         if ( type )
            stack_push( items,  ptr->data.atom->data.string );
         else
            stack_push( items, ptr->data.atom->data.value );
      }

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      ptr2 = &result->data.head;

      if ( type )
      {
         qsort( items->values, items->used, sizeof( void * ), 
                compare_strings );

         for( i = 0; i < items->used; ++i )
         {
            *ptr2 = make_atom_from_string(
               (( struct lstring *)items->values[ i ] )->string,
               (( struct lstring *)items->values[ i ] )->length );
            ( *ptr2 )->next = NULL;
            ptr2 = &( *ptr2 )->next;
         }
      }
      else
      {
         qsort( items->values, items->used, sizeof( void * ), 
                compare_numbers );

         for( i = 0; i < items->used; ++i )
         {
            *ptr2 = make_atom_from_number( *( double *)items->values[ i ] );
            ( *ptr2 )->next = NULL;
            ptr2 = &( *ptr2 )->next;
         }
      }

      stack_free( items );
   }

   return 0;
}

int do_while( char *syntax, struct object *args )
{
   struct object *car, *result;
   int i;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   car = args;
   i = stack->used;

   for( ; ; )
   {
      stack_push( stack, car );

      if ( evaluate() )
      {
         if ( !stop )
            fprintf( stderr, "%s: evaluation of test clause failed.\n", syntax );

         return 1;
      }

      result = stack_pop( stack );

      if (( islist( result->flags ) == 1 && result->data.head == NULL ) ||
            result->data.atom == zero->data.atom ||
            result->data.atom == empty->data.atom )
         break;

      if ( args->next != NULL )
      {
         if ( do_progn( syntax, car->next ))
         {
            if ( !stop )
               fprintf( stderr, "%s: evaluation of body failed.\n", syntax );

            if ( next_iteration )
            {
               next_iteration = 0;
               stop = 0;
               thrown = NULL;
               stack_truncate( stack, stack->used - i );
               continue;
            }

            return 1;
         }

         stack_pop( stack );
      }
   }

   stack_push( stack, result );

   return 0;
}

int do_until( char *syntax, struct object *args )
{
   struct object *car, *result;
   int i;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   car = args;
   i = stack->used;

   for( ; ; )
   {
      stack_push( stack, car );

      if ( evaluate() )
      {
         if ( !stop )
            fprintf( stderr, "%s: evaluation of test clause failed.\n", syntax );

         return 1;
      }

      result = stack_pop( stack );

      if ( !(( islist( result->flags ) == 1 && result->data.head == NULL ) ||
               result->data.atom == zero->data.atom ||
               result->data.atom == empty->data.atom ))
         break;

      if ( args->next != NULL )
      {
         if ( do_progn( syntax, car->next ))
         {
            if ( !stop )
               fprintf( stderr, "%s: evaluation of body failed.\n", syntax );

            if ( next_iteration )
            {
               next_iteration = 0;
               stop = 0;
               thrown = NULL;
               stack_truncate( stack, stack->used - i );
               continue;
            }

            return 1;
         }

         stack_pop( stack );
      }
   }

   stack_push( stack, result );

   return 0;
}

int do_do( char *syntax, struct object *args )
{
   struct object *result;
   int i;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   i = stack->used;

   for( ; ; )
   {
      if ( do_progn( syntax, args ))
      {
         if ( !stop )
            fprintf( stderr, "%s: evaluation of body failed.\n", syntax );

         if ( next_iteration )
         {
            next_iteration = 0;
            stop = 0;
            thrown = NULL;
            stack_truncate( stack, stack->used - i );
            continue;
         }

         return 1;
      }

      result = stack_pop( stack );

      if (( islist( result->flags ) == 1 && result->data.head == NULL ) || 
            result->data.atom == zero->data.atom ||
            result->data.atom == empty->data.atom )
         break;
   }

   stack_push( stack, result );

   return 0;
}

int do_throw( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   thrown = stack_pop( stack );
   stop = 1;

   return 1;
}

int do_catch( char *syntax, struct object *args )
{
   int i;

   i = stack->used;

   if ( do_progn( syntax, args ))
   {
      if ( !stop )
      {
         fprintf( stderr, "%s: evaluation of body failed.\n", syntax );
         return 1;
      }
      else if ( thrown != NULL )
      {
         stack_truncate( stack, stack->used - i );
         stack_push( stack, thrown );
         thrown = NULL;
         stop = 0;
      }
      else
         return 1;
   }

   return 0;
}

int do_die( char *syntax, struct object *args )
{
   if ( args != NULL )
      do_warn( syntax, args );

   stop = 1;
   next_iteration = 0;
   thrown = NULL;

   return 1;
}

int do_stringify( char *syntax, struct object *args )
{
   int i, len;
   struct object *item, *result, *ptr;
   struct stack *myatoms;
   char *tmp;
   struct string *final;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   for( i = 1, item = args; item != NULL; item = item->next, ++i )
   {
      stack_push( stack, item );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 1;
      }

      result = *( struct object **)stack->top;

      if ( islist( result->flags ) == 1 )
      {
         print_err( ERR_ARG_TYPE, syntax, i, ERR_ATOM );
         return 1;
      }
   }

   final = make_string();
   myatoms = make_stack();

   while( --i )
      stack_push( myatoms, stack_pop( stack ));

   while( myatoms->used )
   {
      ptr = stack_pop( myatoms );

      if ( type( ptr->data.atom->flags ) == ATOM_STRING )
      {
         tmp = ptr->data.atom->data.string->string;
         len = ptr->data.atom->data.string->length;

         while( len-- )
            string_append( final, *tmp++ );
      }
      else
      {
         tmp = ptr->data.atom->syntax;
         len = ptr->data.atom->len;

         while( len-- )
            string_append( final, *tmp++ );
      }
   }

   stack_push( stack, make_atom_from_string( final->str, final->used ));

   stack_free( myatoms );
   string_free( final );

   return 0;
}

int do_digitize( char *syntax, struct object *args )
{
   struct object *car;
   double i;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   i = atof( car->data.atom->data.string->string );
   stack_push( stack, make_atom_from_number( i ));

   return 0;
}

int do_intern( char *syntax, struct object *args )
{
   struct object *car;
   char *ptr;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   ptr = car->data.atom->data.string->string;

   if ( *ptr == '\0' )
   {
      fprintf( stderr, "%s: empty string passed as argument.\n", syntax );
      return 1;
   }
   else if ( *ptr < 58 && *ptr > 47 )
   {
      fprintf( stderr, "%s: symbols cannot start with a numerical character.\n",
               syntax );
      return 1;
   }

   for( ++ptr; *ptr; ++ptr )
      if ( *ptr < 48 || 
           *ptr > 122 || 
           ( *ptr > 57 && *ptr < 65 ) ||
           ( *ptr > 90 && *ptr < 95 ) ||
           ( *ptr > 95 && *ptr < 97 ))
         {
            fprintf( stderr, "%s: non-symbol character in argument.\n",
                     syntax );
            return 1;
         }

   stack_push( stack, make_atom_from_symbol( car->data.atom->data.string->string ));

   return 0;
}

int do_additive( char *syntax, struct object *args )
{
   struct object *ptr, *result;
   double total;
   int i, multiply;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   multiply = ( strcmp( syntax, "<PLUS>" ) == 0 ? 0 : 1 );
   total = multiply;
   i = 1;

   for( ptr = args; ptr != NULL; ptr = ptr->next )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 1;
      }

      result = stack_pop( stack );

      if ( islist( result->flags ) == 1 || 
           type( result->data.atom->flags ) != ATOM_NUMBER )
      {
         print_err( ERR_ARG_TYPE, syntax, i, ERR_NUMBER );
         return 1;
      }

      if ( multiply )
         total *= *result->data.atom->data.value;
      else
         total += *result->data.atom->data.value;

      ++i;
   }

   stack_push( stack, make_atom_from_number( total ));

   return 0;
}

int do_subtractive( char *syntax, struct object *args )
{
   struct object *car1, *car2;
   double i1, i2, result;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   i1 = *car1->data.atom->data.value;
   i2 = *car2->data.atom->data.value;

   if ( strcmp( syntax, "<MINUS>" ) == 0 )
      result = i1 - i2;
   else if ( strcmp( syntax, "<DIVIDE>" ) == 0 )
   {
      if ( i2 )
         result = i1 / i2;
      else
      {
         fprintf( stderr, "%s: attempted division by zero.\n", syntax );
         return 1;
      }
   }
   else
   {
      if ( i2 )
         result = i1 - ( i2 * ( int)( i1 / i2 ));
      else
      {
         fprintf( stderr, "%s: attempted division by zero.\n", syntax );
         return 1;
      }
   }

   stack_push( stack, make_atom_from_number( result ));

   return 0;
}

int do_comparative( char *syntax, struct object *args )
{
   double result, i1, i2;
   struct object *car1, *car2;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   i1 = *car1->data.atom->data.value;
   i2 = *car2->data.atom->data.value;

   if ( strcmp( syntax, "<GREATER>" ) == 0 )
      result = ( i1 > i2 ? 1 : 0 );
   else if ( strcmp( syntax, "<GREATER_OR_EQ>" ) == 0 )
      result = ( i1 >= i2 ? 1 : 0 );
   else if ( strcmp( syntax, "<LESSER>" ) == 0 )
      result = ( i1 < i2 ? 1 : 0 );
   else
      result = ( i1 <= i2 ? 1 : 0 );

   stack_push( stack, make_atom_from_number( result ));

   return 0;
}

int do_abs( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   stack_push( stack, make_atom_from_number( abs( *car->data.atom->data.value )));

   return 0;
}

int do_char( char *syntax, struct object *args )
{
   struct object *car;
   char s[ 2 ];
   int i;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   i = *car->data.atom->data.value;

   if ( i < 0 || i > 255 )
   {
      fprintf( stderr, "%s: argument out of range: %d\n", 
               syntax, i );
      return 1;
   }

   s[ 0 ] = ( char )i;
   s[ 1 ] = '\0';

   stack_push( stack, make_atom_from_string( s, 1 ));

   return 0;
}

int do_code( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( car->data.atom->data.string->length == 0 )
   {
      fprintf( stderr, "%s: argument is empty string.\n", syntax );
      return 1;
   }

   stack_push( stack, 
      make_atom_from_number( ( unsigned char)car->data.atom->data.string->string[ 0 ] ));

   return 0;
}

int do_open( char *syntax, struct object *args )
{
   DB *new_db;
   mode_t mode;

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   mode = getmode( setmode( "0600" ), 0 );

AGAIN:
   if (( new_db = dbopen( NULL, O_EXCL | O_EXLOCK | O_RDWR | O_CREAT, mode, 
                          DB_RECNO, NULL )) == NULL )
   {
      if ( errno == EINTR )
         goto AGAIN;

      fprintf( stderr, "%s: %s.\n", syntax, strerror( errno ));
      return 1;
   }

   stack_push( buffer_stack, new_db );
   buffer = new_db;

   bookmarks = 
      ( struct hash_elt **)memory( HASH_SIZE * sizeof( struct hash_elt * ));
   bzero( bookmarks, ( HASH_SIZE * sizeof( struct hash_elt * )));

   stack_push( bookmark_stack, bookmarks );

   stack_push( stack, make_atom_from_number( buffer_stack->used - 1 ));

   return 0;
}

int do_close( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer has been opened.\n", syntax );
      return 1;
   }

   if ( buffer->close( buffer ) )
   {
      fprintf( stderr, "%s: db->close: %s", syntax, strerror( errno ));
      return 1;
   }
   else
   {
      int i;

      for( i = 0; i < buffer_stack->used; ++i )
         if ( buffer_stack->values[ i ] == buffer )
         {
            buffer_stack->values[ i ] = NULL;
            hash_free( bookmark_stack->values[ i ] );
            free( bookmark_stack->values[ i ] );
            bookmark_stack->values[ i ] = NULL;
            break;
         }

      buffer = NULL;

      for( i = buffer_stack->used - 1; i >= 0; --i )
      {
         buffer = ( DB *)buffer_stack->values[ i ];
         if ( buffer != NULL )
         {
            bookmarks = bookmark_stack->values[ i ];
            break;
         }
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_insert( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3;
   int arg3, flag = 0, real_line;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   car3 = stack_pop( stack );
   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   key_data = *car1->data.atom->data.value;
   real_line = key_data;

   if ( key_data < 0 )
   {
      fprintf( stderr, "%s: argument 1 < 0.\n", syntax );
      return 1;
   }

   arg3 = *car3->data.atom->data.value;

   if ( arg3 == 0 )
      flag = R_SETCURSOR;
   else if ( arg3 > 0 )
   {
      flag = R_IAFTER;
      real_line += 1;
   }
   else if ( arg3 < 0 )
   {
      flag = R_IBEFORE;
      real_line -= 1;
   }

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   dbt_value.data = car2->data.atom->data.string->string;
   dbt_value.size = car2->data.atom->data.string->length + 1;

   if ( buffer->put( buffer, &dbt_key, &dbt_value, flag ) < 0 )
   {
      fprintf( stderr, "%s: db->put: %s.\n", syntax, strerror( errno ));
      return 1;
   }
   else
      stack_push( stack, make_atom_from_number( 1 ));

   if ( real_line == key_data )
      delete_bookmarks( real_line, real_line );
   else
      adjust_bookmarks( real_line, 1 );

   return 0;
}

int do_delete( char *syntax, struct object *args )
{
   struct object *car;
   int result, arg1;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   arg1 = *car->data.atom->data.value;

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );
   key_data = arg1;

   if (( result = buffer->del( buffer, &dbt_key, 0 )) < 0 )
   {
      fprintf( stderr, "%s: delete: db->del: %s.\n", syntax, strerror( errno ));
      stack_push( stack, make_atom_from_number( 0 ));
      return 1;
   }
   else if ( result )
   {
      fprintf( stderr, "%s: index does not exist: %d.\n", syntax, 
               key_data );
      stack_push( stack, make_atom_from_number( 0 ));
      return 1;
   }

   delete_bookmarks( key_data, key_data );
   adjust_bookmarks( key_data, -1 );

   stack_push( stack, make_atom_from_number( 1 )); 

   return 0;
}

int do_lastline( char *syntax, struct object *args )
{
   int result;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   if (( result = buffer->seq( buffer, &dbt_key, &dbt_value, R_LAST )) == -1 )
   {
      fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
      stack_push( stack, make_atom_from_number( 0 ));
   }
   else if ( result == 1 )
      stack_push( stack, make_atom_from_number( 0 ));
   else
      stack_push( stack, make_atom_from_number( *( int *)dbt_key.data ));

   return 0;
}

int do_retrieve( char *syntax, struct object *args )
{
   struct object *car;
   int result;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   key_data = *car->data.atom->data.value;

   if ( key_data <= 0 )
   {
      fprintf( stderr, "%s: index <= 0.\n", syntax );
      return 1;
   }

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   if (( result = buffer->get( buffer, &dbt_key, &dbt_value, 0 )) < 0 )
   {
      fprintf( stderr, "%s: db->get: %s.\n", syntax, strerror( errno ));
      return 1;
   }
   else if ( result == 1 )
   {
      fprintf( stderr, "%s: db->get: key does not exist.\n", syntax );
      return 1;
   }

   stack_push( stack, make_atom_from_string( dbt_value.data, dbt_value.size - 1 ));

   return 0;
}

int exchange_data( char *syntax, int fd, int begin, int end )
{
   struct stack *old_lines, *new;
   struct string *s;
   int i, out, result;
   fd_set in_set, out_set;

   old_lines = make_stack();
   new = make_stack();
   s = make_string();

   if ( end )
   {
      out = 1;

      for( i = end; i >= begin; --i )
      {
         char *line;

         dbt_key.data = &key_data;
         dbt_key.size = sizeof( recno_t );
         key_data = i;

         if (( result = buffer->get( buffer, &dbt_key, &dbt_value, 0 )) < 0 )
         {
            fprintf( stderr, "%s: db->get: %s.\n", syntax, strerror( errno ));

            while( old_lines->used )
               free( stack_pop( old_lines ));

            stack_free( old_lines );
            stack_free( new );
            string_free( s );
            return 1;
         }
         else if ( result == 1 )
         {
            fprintf( stderr, "%s: db->get: key does not exist.\n", syntax );

            while( old_lines->used )
               free( stack_pop( old_lines ));

            stack_free( old_lines );
            stack_free( new );
            string_free( s );
            return 1;
         }

         line = str_dup( ( char *)dbt_value.data, dbt_value.size );
         stack_push( old_lines, line );
      }

      delete_bookmarks( begin, end );
   }
   else
   {
      shutdown( fd, SHUT_WR );
      out = 0;
   }

   i = fd + 1;

   for( ; ; )
   {
      char c[ 128 ];

      FD_ZERO( &in_set );
      FD_ZERO( &out_set );

      FD_SET( fd, &in_set );
      if ( out )
         FD_SET( fd, &out_set );

      result = select( i, &in_set, &out_set, NULL, NULL );

      if ( result < 0 )
      {
         if ( errno == EINTR )
            continue;

         close( fd );
         fprintf( stderr, "%s: select: %s.\n", syntax, strerror( errno ));

         while( old_lines->used )
            free( stack_pop( old_lines ));

         while( new->used )
            free( stack_pop( new ));

         stack_free( old_lines );
         stack_free( new );
         string_free( s );
         return 1;
      }

      if ( FD_ISSET( fd, &in_set ))
      {
         result = read( fd, c, sizeof( c ) - 1 );
         if ( result < 0 )
         {
            close( fd );
            fprintf( stderr, "%s: read: %s.\n", syntax, strerror( errno ));

            while( old_lines->used )
               free( stack_pop( old_lines ));

            while( new->used )
               free( stack_pop( new ));

            stack_free( old_lines );
            stack_free( new );
            string_free( s );
            return 1;
         }
         else if ( result == 0 )
         {
            ( out ?  close( fd ) : shutdown( fd, SHUT_RD ));
            break;
         }
         else
         {
            char *ptr;

            c[ result ] = '\0';

            for( ptr = c; *ptr; ++ptr )
               if ( *ptr == '\n' )
               {
                  string_append( s, *ptr );
                  stack_push( new, str_dup( s->str, s->used ));
                  string_truncate( s );
               }
               else
                  string_append( s, *ptr );
         }
      }

      if ( out && FD_ISSET( fd, &out_set ))
      {
         char *line = NULL, *ptr = NULL;

         if ( ptr == NULL || *ptr == '\0' )
            ptr = line = ( char *)stack_pop( old_lines );

         result = write( fd, ptr, strlen( ptr ));
         ptr += result;

         if ( result < 0 )
         {
            close( fd );
            fprintf( stderr, "%s: write: %s.\n", syntax, strerror( errno ));

            while( old_lines->used )
               free( stack_pop( old_lines ));

            while( new->used )
               free( stack_pop( new ));

            stack_free( old_lines );
            stack_free( new );
            string_free( s );
            return 1;
         }

         if ( *ptr == '\0' )
         {
            free( line );

            if ( old_lines->used == 0 )
            {
               shutdown( fd, SHUT_WR );
               line = NULL;
               ptr = NULL;
               out = 0;
            }
         }
      }
   }

   if ( s->used )
      stack_push( new, str_dup( s->str, s->used ));

   wait( &result );

   if ( WIFEXITED( result ))
      result = WEXITSTATUS( result );
   else if ( WIFSIGNALED( result ))
      result = WTERMSIG( result );

   while( old_lines->used )
      free( stack_pop( old_lines ));

   stack_free( old_lines );
   string_free( s );

   if ( result )
   {
      while( new->used )
         free( stack_pop( new ));

      stack_free( new );

      stack_push( stack, make_atom_from_number( 0 ));
      return 0;
   }

   if ( end )
   {
      delete_bookmarks( begin, end );
      adjust_bookmarks( end, ( begin - end ) - 1 );

      for( i = begin; i <= end; ++i )
      {
         key_data = begin;

         if ( buffer->del( buffer, &dbt_key, 0 ))
         {
            while( new->used )
               free( stack_pop( new ));

            stack_free( new );

            fprintf( stderr, "%s: db->del: %s.\n", syntax, strerror( errno ));
            return 1;
         }
      }
   }

   stack_push( stack, make_atom_from_number( new->used ));

   if ( new->used )
   {
      int flag, i, altered;

      if ( end )
      {
         flag = R_IBEFORE;
         altered = begin - 1;
      }
      else
      {
         flag = R_IAFTER;
         altered = begin;
      }

      if ( !end && !begin )
      {
         flag = R_IBEFORE;
         altered = 0;
         begin = 1;
      }

      adjust_bookmarks( altered, new->used );

      key_data = begin;

      for( i = 0; i < new->used; ++i )
      {
         char *line;

         line = ( char *)new->values[ i ];

         dbt_key.data = &key_data;
         dbt_key.size = sizeof( recno_t );
         dbt_value.data = line;
         dbt_value.size = strlen( line ) + 1;

         if ( buffer->put( buffer, &dbt_key, &dbt_value, flag ) < 0 )
         {
            free( line );

            while( new->used )
               free( stack_pop( new ));

            stack_free( new );

            fprintf( stderr,"%s: dbt->put: %s.\n", syntax, strerror( errno ));
            return 1;
         }

         ++key_data;

         free( line );
      }
   }

   stack_free( new );

   return 0;
}

int do_filter( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3;
   char *name, *exec_args[ 4 ];
   int pid, beginning, ending, result;
   int fd[ 2 ];

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   car3 = stack_pop( stack );
   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   beginning = *car1->data.atom->data.value;
   ending = *car2->data.atom->data.value;

   if ( beginning > ending )
   {
      int temp = beginning;
      beginning = ending;
      ending = temp;
   }

   name = car3->data.atom->data.string->string;

   exec_args[ 0 ] = "/bin/sh";
   exec_args[ 1 ] = "-c";
   exec_args[ 2 ] = name;
   exec_args[ 3 ] = NULL;

   if ( socketpair( PF_LOCAL, SOCK_STREAM, 0, fd ))
   {
      fprintf( stderr, "%s: socketpair: %s.\n", syntax, strerror( errno ));
      return 1;
   }

   switch( pid = fork() )
   {
      case -1:
         fprintf( stderr, "%s: fork: %s.\n", syntax, strerror( errno ));
         return 1;

      case 0:
         if ( dup2( fd[ 0 ], STDIN_FILENO ) < 0 ||
              dup2( fd[ 0 ], STDOUT_FILENO ) < 0 )
         {
            fprintf( stderr, "%s (child): dup2: %s.\n", syntax, strerror( errno ));
            _exit( 1 );
         }

         close( fd[ 0 ] );
         close( fd[ 1 ] );

         execv( exec_args[ 0 ], exec_args );
         _exit( 1 );

      default:
         close( fd[ 0 ] );
   }

   signal( SIGCHLD, SIG_IGN );
   result = exchange_data( syntax, fd[ 1 ], beginning, ending );
   signal( SIGCHLD, sigchld_handler );

   return result;
}

int do_write( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3, *car4, *car5;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   car1 = args;
   car2 = car1->next;

   if ( car2 == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, -1 );
      return 1;
   }

   car3 = car2->next;

   if ( car3 == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 3, -1 );
      return 1;
   }

   car4 = car3->next;

   if ( car4 == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 4, -1 );
      return 1;
   }

   car5 = car4->next;

   if ( car5 != NULL && car5->next != NULL )
   {
      print_err( ERR_MORE_ARGS, syntax, 5, -1 );
      return 1;
   }

   stack_push( stack, car1 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 1, -1 );
      return 1;
   }

   stack_push( stack, car2 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 2, -1 );
      return 1;
   }

   stack_push( stack, car3 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 3, -1 );
      return 1;
   }

   stack_push( stack, car4 );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 4, -1 );
      return 1;
   }

   if ( car5 != NULL )
   {
      stack_push( stack, car5 );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, 5, -1 );
         return 1;
      }

      car5 = stack_pop( stack );

      if ( islist( car5->flags ) == 1 || 
           type( car5->data.atom->flags ) != ATOM_NUMBER )
      {
         print_err( ERR_ARG_TYPE, syntax, 5, ERR_NUMBER );
         return 1;
      }
   }

   car4 = stack_pop( stack );

   if ( islist( car4->flags ) == 1 || 
        type( car4->data.atom->flags ) != ATOM_NUMBER )
   {
      print_err( ERR_ARG_TYPE, syntax, 4, -1 );
      return 1;
   }

   car3 = stack_pop( stack );

   if ( islist( car3->flags ) == 1 || 
        type( car3->data.atom->flags ) != ATOM_STRING )
   {
      print_err( ERR_ARG_TYPE, syntax, 3, -1 );
      return 1;
   }

   car2 = stack_pop( stack );

   if ( islist( car2->flags ) == 1 || 
        type( car2->data.atom->flags ) != ATOM_NUMBER )
   {
      print_err( ERR_ARG_TYPE, syntax, 2, -1 );
      return 1;
   }

   car1 = stack_pop( stack );

   if ( islist( car1->flags ) == 1 || 
        type( car1->data.atom->flags ) != ATOM_NUMBER )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, -1 );
      return 1;
   }

   {
      char *name, *ptr;
      mode_t mode;
      int arg1, arg2, fd, i, flags, result, escape, create_empty;

      arg1 = *car1->data.atom->data.value;
      arg2 = *car2->data.atom->data.value;

      create_empty = 0;

      if ( !arg1 && !arg2 )
         create_empty = 1;

      escape = 0;
      name = car3->data.atom->data.string->string;

      for( ptr = name; *ptr; ++ptr )
      {
         if ( *ptr == '\\' )
         {
            escape ^= 1;
            continue;
         }

         if ( escape && ( *ptr == 'b' || *ptr == 't' ))
         {
            char *d = ptr;

            *( ptr - 1) = ( *ptr == 't' ? '\t' : ' ' );
            ++ptr;
            bcopy( ptr, d, strlen( ptr ) + 1 );
            ptr -= 2;
         }

         escape = 0;
      }

      if ( car5 == NULL || ( *car5->data.atom->data.value ) == 0 )
         flags = ( O_CREAT | O_WRONLY );
      else
         flags = ( O_CREAT | O_APPEND | O_WRONLY );

      if ( *car4->data.atom->data.value == 1 )
         flags |= O_EXLOCK | O_NONBLOCK;

      mode = getmode( setmode( "0600" ), 0 );

   AGAIN:
      if (( fd = open( name, flags, mode )) < 0 )
      {
         if ( errno == EINTR )
            goto AGAIN;

         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }
            
      /*
       * Truncation must performed AFTER opening the file, because if
       * specified by the O_TRUNC flag to open(), the file will be cleared,
       * even if the function fails due to the existence of an exclusive
       * lock for the file.  This is correct, but stupid, UNIX semantics. 
       */

      if ( car5 == NULL || *car5->data.atom->data.value == 0 )
         if ( ftruncate( fd, 0 ))
         {
            close( fd );
            stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
            return 0;
         }

      if ( create_empty )
         stack_push( stack, make_atom_from_number( 0 ));
      else 
      {
         char output_buffer[ 102400 ], *ptr;
         int room = sizeof( output_buffer );

         if ( arg1 > arg2 )
         {
            int tmp;

            tmp = arg1;
            arg1 = arg2;
            arg2 = tmp;
         }

         ptr = output_buffer;

         for( i = arg1; i <= arg2; ++i )
         {
            dbt_key.data = &key_data;
            dbt_key.size = sizeof( recno_t );
            key_data = i;

            if (( result = buffer->get( buffer, &dbt_key, &dbt_value, 0 )) < 0 )
            {
               close( fd );
               stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
               return 0;
            }
            else if ( result == 1 )
            {
               close( fd );
               stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
               return 0;
            }

            if (( room - ( int )( dbt_value.size - 1 )) < 0 )
            {
               if ( write( fd, output_buffer, ptr - output_buffer ) < 0 )
               {
                  close( fd );
                  stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
                  return 0;
               }

               ptr = output_buffer;
               room = sizeof( output_buffer );
            }

            bcopy( dbt_value.data, ptr, dbt_value.size - 1 );
            ptr += dbt_value.size - 1;
            room -= dbt_value.size - 1;
         }

         if ( write( fd, output_buffer, ptr - output_buffer ) < 0 )
         {
            close( fd );
            stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
            return 0;
         }

         stack_push( stack, make_atom_from_number( arg2 - arg1 + 1 ));
      }

      close( fd );
   }

   return 0;
}

int do_read( char *syntax, struct object *args )
{
   struct object *car1, *car2;
   struct string *s;
   int fd, arg1, i, escape, flags, count = 0, altered = 0, quantity;
   char *name, *ptr;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   arg1 = *car1->data.atom->data.value;

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   if ( buffer->seq( buffer, &dbt_key, &dbt_value, R_LAST ) == -1 )
   {
      fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
      stack_push( stack, make_atom_from_number( 0 ));
   }

   if ( *( int *)dbt_key.data )
   {
      if ( arg1 )
      {
         flags = R_IAFTER;
         altered = key_data;
      }
      else
      {
         flags = R_IBEFORE;
         altered = key_data - 1;
      }
   }           
   else
      flags = R_SETCURSOR;

   key_data = ( arg1 ? arg1 : 1 );

   name = car2->data.atom->data.string->string;

   escape = 0;

   for( ptr = name; *ptr; ++ptr )
   {
      if ( *ptr == '\\' )
      {
         escape ^= 1;
         continue;
      }

      if ( escape && ( *ptr == 'b' || *ptr == 't' ))
      {
         char *d = ptr;
         *( ptr - 1) = ( *ptr == 't' ? '\t' : ' ' );
         ++ptr;
         bcopy( ptr, d, strlen( ptr ) + 1 );
         ptr -= 2;
      }

      escape = 0;
   }

AGAIN:
   if (( fd = open( name, O_RDONLY )) < 0 )
   {
      if ( errno == EINTR )
         goto AGAIN;

      switch( errno )
      {
         case ENOENT:
            stack_push( stack, make_atom_from_number( -1 ));
            break;

         case EACCES:
            stack_push( stack, make_atom_from_number( -2 ));
            break;

         default:
            stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
            break;
      }

      return 0;
   }

   s = make_string();

   quantity = 0;

   for( i = 0; ; ++i )
   {
      int result;
      char *ptr;
      char input_buffer[ 102400 ];

   READ:
      result = read( fd, input_buffer, sizeof( input_buffer ) - 1 );

      if ( result < 0 )
      {
         fprintf( stderr, "%s: read: %s.\n", syntax, strerror( errno ));
         break;
      }
      else if ( result == 0 )
      {
         if ( !count && s->used )
            ++count;

         if ( s->used )
         {
            dbt_value.data = s->str;
            dbt_value.size = s->used + 1;

            dbt_key.data = &key_data;
            dbt_key.size = sizeof( recno_t );

            if ( buffer->put( buffer, &dbt_key, &dbt_value, flags ) < 0 )
            {
               fprintf( stderr, "%s: db->put: %s.\n", syntax, 
                        strerror( errno ));
               string_free( s );
               close( fd );
               return 1;
            }

            ++quantity;
         }

         break;
      }
      else 
         input_buffer[ result ] = '\0';

      ptr = input_buffer;

      do
      {
         char *ptr2, *tmp;

         ptr2 = ptr;

         if (( ptr = strchr( ptr, '\n' )) == NULL )
         {
            tmp = ptr2;
            while( *tmp )
               string_append( s, *tmp++ );
            goto READ;
         }

         *ptr = '\0';

         tmp = ptr2;
         while( *tmp )
            string_append( s, *tmp++ );
         string_append( s, '\n' );

         dbt_value.data = s->str;
         dbt_value.size = s->used + 1;

         dbt_key.data = &key_data;
         dbt_key.size = sizeof( recno_t );

         if ( buffer->put( buffer, &dbt_key, &dbt_value, flags ) < 0 )
         {
            fprintf( stderr, "%s: db->put: %s.\n", syntax, 
                     strerror( errno ));
            string_free( s );
            close( fd );
            return 1;
         }

         string_truncate( s );

         ++quantity;
         ++count;
         ++key_data;
      }
      while( *++ptr );
   }

   string_free( s );
   close( fd );
   adjust_bookmarks( altered, quantity );

   stack_push( stack, make_atom_from_number( count ));

   return 0;
}

int do_empty( char *syntax, struct object *args )
{
   int i, last;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   if (( i = buffer->seq( buffer, &dbt_key, &dbt_value, R_LAST )) == 1 )
   {
      stack_push( stack, make_atom_from_number( 0 ));
      return 0;
   }
   else if ( i < 0 )
   {
      fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
      return 1;
   }

   last = *( int *)dbt_key.data;

   if ( last == 0 )
   {
      stack_push( stack, make_atom_from_number( 1 ));
      return 0;
   }

   for( i = 0; i < 27; ++i )
      bookmarks[ i ] = 0;

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );
   key_data = 1;

   for( i = 1; i <= last; ++i )
   {
      int result;

      if (( result = buffer->del( buffer, &dbt_key, 0 )) < 0 )
      {
         fprintf( stderr, "%s: db->del: %s.\n", syntax, strerror( errno ));
         return 1;
      }
      else if ( result )
      {
         fprintf( stderr, "%s: index does not exist: %d.\n", syntax, i );
         return 1;
      }
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_slice( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3, *car4, *car5;
   int arg1, arg2, arg3, arg4, arg5;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)5 );

   if ( check_args( syntax, args ))
      return 1;

   car5 = stack_pop( stack );
   car4 = stack_pop( stack );
   car3 = stack_pop( stack );
   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   arg1 = *car1->data.atom->data.value;
   arg2 = *car2->data.atom->data.value;
   arg3 = *car3->data.atom->data.value;
   arg4 = *car4->data.atom->data.value;
   arg5 = *car5->data.atom->data.value;

   if ( arg4 <= 0 )
   {
      fprintf( stderr, "%s: tabsize specifier <= 0.\n", syntax );
      return 1;
   }

   {
      struct string *s = NULL, *e = NULL;
      char *ptr;
      recno_t i;
      int offset, *offsets = NULL, result;

      dbt_key.data = &key_data;
      dbt_key.size = sizeof( recno_t );
      key_data = arg1;

      if (( result = buffer->get( buffer, &dbt_key, &dbt_value, 0 )) < 0 )
      {
         fprintf( stderr, "%s: db->get: %s.\n", syntax, strerror( errno ));
         return 1;
      }
      else if ( result == 1 )
      {
         fprintf( stderr, "%s: db->get: key does not exist.\n", syntax );
         return 1;
      }

      if ( arg5 && dbt_value.size > 1 )
         offsets = ( int *)memory( sizeof( int ) * ( dbt_value.size - 1 ));
      else
         s = make_string();

      e = make_string();

      if ( dbt_value.size > 1 )
      {
         ptr = ( char *)dbt_value.data;
         offset = 0;

         for( i = 0; i < dbt_value.size - 1; ++i )
         {
            if ( *ptr == '\t' )
            {
               int spaces;

               spaces = arg4 - ( i + offset ) % arg4;
               offset += spaces - 1;

               if ( !arg5 )
                  while( spaces-- )
                     string_append( e, ' ' );
            }
            else if ( !arg5 )
               string_append( e, *ptr );

            ++ptr;

            if ( arg5 )
               offsets[ i ] = offset;
         }
      }

      if ( arg5 )
      {
         struct object *result;
         int length;

         result = make_object();
         stack_push( stack, result );
         setlist( result->flags );

         length = ( dbt_value.size - 1 ) - arg2;
         if ( arg3 )
            length = MIN( arg3, length );

         result->data.head = make_atom_from_number( length );

         result->data.head->next = 
            make_atom_from_number(( length ? offsets[ --length ] : 0 ));

         if ( dbt_value.size > 1 ) 
            free( offsets );
      }
      else
      {
         if ( arg2 >= e->used )
         {
            stack_push( stack, make_atom_from_string( "", 0 ));
            string_free( s );
            string_free( e );
            return 0;
         }

         ptr = &e->str[ arg2 ];
         if ( arg3 )
            arg3 += arg2;

         result = ( arg3 ? MIN( arg3, e->used ) : e->used );

         for( i = arg2; i < result; ++i )
            string_append( s, *ptr++ );

         stack_push( stack, make_atom_from_string( s->str, s->used ));
         string_free( s );
      }

      string_free( e );
   }

   return 0;
}

int do_find( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3, *car4, *car5;
   char *old_ptr;
   int limit, last, old_arg3, i, j, flags,
      found, inc, end, start, result, old_result,
      arg1, arg2, arg3, arg5;
   regmatch_t matches, old_matches;
   regex_t *r;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_REGEXP );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)5 );

   if ( check_args( syntax, args ))
      return 1;

   car5 = stack_pop( stack );
   car4 = stack_pop( stack );
   car3 = stack_pop( stack );
   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   arg5 = *car5->data.atom->data.value;
   arg1 = *car1->data.atom->data.value;

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   if ( buffer->seq( buffer, &dbt_key, &dbt_value, R_LAST ) == -1 )
   {
      fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
      stack_push( stack, make_atom_from_number( 0 ));
   }

   last = *( int *)dbt_key.data;

   if ( arg1 < 0 )
   {
      end = 0;
      inc = -1;
   }
   else
   {
      end = last + 1;
      inc = 1;
   }

   arg2 = *car2->data.atom->data.value;
   start = arg2;

   arg3 = *car3->data.atom->data.value;

   found = 0;
   limit = arg3;

   r = car4->data.atom->data.regexp;

   for( j = 0; j < 2; ++j )
   {
      for( i = start; i != end; i += inc )
      {
         char *ptr, *temp;

         dbt_key.data = &key_data;
         dbt_key.size = sizeof( recno_t );
         key_data = i;

         if (( result = buffer->get( buffer, &dbt_key, &dbt_value, 0 )) < 0 )
         {
            fprintf( stderr, "%s: db->get: %s.\n", 
                     syntax, strerror( errno ));
            return 1;
         }
         else if ( result == 1 )
         {
            fprintf( stderr, "%s: db->get: key does not exist: %d.\n",
                     syntax, i );
            return 1;
         }

         temp = ( char *)dbt_value.data;
         temp = str_dup( temp, dbt_value.size );

         if (( ptr = strchr( temp, '\n' )) != NULL )
            *ptr = '\0';

         ptr = temp;

         if ( i == arg2 && 
            ( arg3 < 0 || ( arg3 && arg3 > dbt_value.size - 2 )))
         {
            fprintf( stderr, "%s: argument 3 out of range.\n", syntax );
            return 1;
         }

         result = REG_NOMATCH;

         arg3 = 0;
         old_ptr = NULL;
         matches.rm_eo = 0;
         flags = 0;

         do
         {
            old_arg3 = arg3;
            arg3 += matches.rm_eo;
            old_result = result;
            old_matches = matches;

            if ( ptr == old_ptr )
               break;

            old_ptr = ptr;

            result = regexec( r, ptr, 1, &matches, flags );

            if ( !result )
            {
               if ( i == arg2 )
               {
                  if (( arg1 < 0 && arg3 + matches.rm_so >= limit ) ||
                      ( arg1 > 0 && arg3 + matches.rm_so > limit ))
                     break;
               }
               else if ( arg1 > 0 )
                  break;

               ptr += matches.rm_eo;
            }

            flags = REG_NOTBOL;
         }
         while( !result );

         free( temp );

         if ( arg1 < 0 )
         {
            result = old_result;
            matches = old_matches;
            arg3 = old_arg3;
         }
         else if ( i == arg2 )
         {
            if ( arg3 + matches.rm_so <= limit )
               continue;

            if ( limit + 1 == dbt_value.size - 1 &&
                 limit + 1 == arg3 + matches.rm_so )
               continue;
         }

         if ( result )
         {
            if ( result == REG_NOMATCH )
               continue;

            {
               char err[ 80 ];

               regerror( result, r, err, sizeof( err ));
               fprintf( stderr, "%s: regexec: %s.\n", syntax, err );
               return 1;
            }
         }

         found = 1;
         goto LIST;
      }

      if ( !arg5 )
         break;

      if ( arg1 > 0 )
      {
         start = 1;
         end = last + 1; 
      }
      else
      {
         start = last;
         end = 0;
      }

      arg2 = 0;
   }

LIST:
   car1 = make_object();
   stack_push( stack, car1 );
   setlist( car1->flags );

   if ( !found )
   {
      car1->data.head = make_atom_from_number( 0 );
      car1->data.head->next = make_atom_from_number( 0 );
      car1->data.head->next->next = make_atom_from_number( 0 );
   }     
   else
   {
      int len = matches.rm_eo - matches.rm_so;

      if (( i == arg2 && arg1 > 0 ) || arg1 < 0 )
         matches.rm_so += arg3;

      if ( matches.rm_so > 0 && matches.rm_so == dbt_value.size - 1 )
         --matches.rm_so;

      car1->data.head = make_atom_from_number( i );
      car1->data.head->next = make_atom_from_number( matches.rm_so );
      car1->data.head->next->next = make_atom_from_number( len );
   }

   return 0;
}

int do_input( char *syntax, struct object *args )
{
   struct object *car1, *car2;
   int pipe, arg1, last, result, escape;
   char *name, *ptr;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   if ( buffer->seq( buffer, &dbt_key, &dbt_value, R_LAST ) == -1 )
   {
      fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
      stack_push( stack, make_atom_from_number( 0 ));
      return 1;
   }

   last = *( int *)dbt_key.data;

   arg1 = *car1->data.atom->data.value;

   if ( arg1 < 0 || arg1 > last )
   {
      fprintf( stderr, "%s: argument 1 out of range.\n", syntax );
      return 1;
   }

   name = car2->data.atom->data.string->string;
   escape = 0;

   for( ptr = name; *ptr; ++ptr )
   {
      if ( *ptr == '\\' )
      {
         escape ^= 1;
         continue;
      }

      if ( escape && ( *ptr == 'b' || *ptr == 't' ))
      {
         char *d = ptr;
         *( ptr - 1) = ( *ptr == 't' ? '\t' : ' ' );
         ++ptr;
         bcopy( ptr, d, strlen( ptr ) + 1 );
         ptr -= 2;
      }

      escape = 0;
   }

   pipe = pipe_open( syntax, name, 0, 0 );

   if ( pipe == -1 )
   {
      stack_push( stack, make_atom_from_number( 0 ));
      return 0;
   }

   signal( SIGCHLD, SIG_IGN );
   result = exchange_data( syntax, pipe, arg1, 0 );
   signal( SIGCHLD, sigchld_handler );

   return result;
}

int do_output( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3;
   char *name, *ptr;
   int i, result, escape, arg1, arg2, pipe;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   car3 = stack_pop( stack );
   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   arg1 = *car1->data.atom->data.value;
   arg2 = *car2->data.atom->data.value;

   if ( arg1 > arg2 )
   {
      int tmp;

      tmp = arg1;
      arg1 = arg2;
      arg2 = tmp;
   }

   name = car3->data.atom->data.string->string;
   escape = 0;

   for( ptr = name; *ptr; ++ptr )
   {
      if ( *ptr == '\\' )
      {
         escape ^= 1;
         continue;
      }

      if ( escape && ( *ptr == 'b' || *ptr == 't' ))
      {
         char *d = ptr;
         *( ptr - 1) = ( *ptr == 't' ? '\t' : ' ' );
         ++ptr;
         bcopy( ptr, d, strlen( ptr ) + 1 );
         ptr -= 2;
      }

      escape = 0;
   }

   pipe = pipe_open( syntax, name, 1, 0 );

   if ( pipe == -1 )
   {
      stack_push( stack, make_atom_from_number( 0 ));
      return 0;
   }

   for( i = arg1; i <= arg2; ++i )
   {
      int len, written;
      char *current;

      dbt_key.data = &key_data;
      dbt_key.size = sizeof( recno_t );
      key_data = i;

      if (( result = buffer->get( buffer, &dbt_key, &dbt_value, 0 )) < 0 )
      {
         close( pipe );
         fprintf( stderr, "%s: db->get: %s.\n", syntax, strerror( errno ));
         return 1;
      }
      else if ( result == 1 )
      {
         close( pipe );
         fprintf( stderr, "%s: db->get: key does not exist.\n", syntax );
         return 1;
      }

      current = ( char *)dbt_value.data;
      written = 0;

      do
      {
         current += written;
         len = strlen( current );

         if (( written = write( pipe, current, len )) < 0 )
         {
            close( pipe );

            if ( errno == EPIPE )
            {
               stack_push( stack, make_atom_from_number( i - arg1 ));
               return 0;
            }

            fprintf( stderr, "%s: %s.\n", syntax, strerror( errno ));
            return 1;
         }
      }
      while( written < len );
   }

   close( pipe );
   stack_push( stack, make_atom_from_number( i - arg1 ));

   return 0;
}

int do_system( char *syntax, struct object *args )
{
   struct object *car;
   int result;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   result = system( car->data.atom->data.string->string );
   stack_push( stack, make_atom_from_number( result ));

   return 0;
}

int do_maxidx( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   stack_push( stack, make_atom_from_number( INT_MAX ));

   return 0;
}

int do_chdir( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( chdir( car->data.atom->data.string->string ))
   {
      stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
      return 0;
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_bound( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_ATOM );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( lookup_local( car->data.atom->id ) == NULL &&
        lookup_binding( car->data.atom->id ) == NULL )
      stack_push( stack, make_atom_from_number( 0 ));
   else
      stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_buffer( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( buffer == NULL )
   {
      stack_push( stack, make_atom_from_number( -1 ));
      return 0;
   }

   {
      int i;

      for( i = 0; i < buffer_stack->used; ++i )
         if ( buffer == buffer_stack->values[ i ] )
            break;

      stack_push( stack, make_atom_from_number( i ));
   }

   return 0;
}

int do_buffers( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object **ptr, *result;
      int i;

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      ptr = &result->data.head;

      for( i = 0; i < buffer_stack->used; ++i )
         if ( buffer_stack->values[ i ] != NULL )
         {
            *ptr = make_atom_from_number( i );
            ptr = &( *ptr )->next;
         }
   }

   return 0;
}

int do_switch( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      int i;

      car = stack_pop( stack );
      i = *car->data.atom->data.value;

      if ( i < 0 )
      {
         fprintf( stderr, "%s: negative buffer number: %d.\n", syntax, i );
         return 1;
      }

      if ( i > buffer_stack->used - 1 ||
           buffer_stack->values[ i ] == NULL )
      {
         fprintf( stderr, "%s: buffer %d is not open.\n", syntax, i );
         return 1;
      }

      buffer = ( DB *)buffer_stack->values[ i ];
      bookmarks = ( struct hash_elt **)bookmark_stack->values[ i ];

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_version( char *syntax, struct object *args )
{
   struct object *result;

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   result = make_object();
   setlist( result->flags );
   stack_push( stack, result );

   result->data.head = make_atom_from_number( VERSION_MAJOR );
   result->data.head->next = make_atom_from_number( VERSION_MINOR );

   return 0;
}

int do_gensym( char *syntax, struct object *args )
{
   char name[ 64 ];

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   snprintf( name, sizeof( name ), "<GENSYM#%d>", gensym_counter++ );
   stack_push( stack, make_atom_from_symbol( name ));

   return 0;
}

int do_libdir( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   stack_push( stack, make_atom_from_string( LISPDIR, strlen( LISPDIR )));

   return 0;
}

int do_substring( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3;
   char *name;
   int len, arg2, arg3;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   car3 = stack_pop( stack );
   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   arg2 = *car2->data.atom->data.value;
   arg3 = *car3->data.atom->data.value;

   len = car1->data.atom->data.string->length;

   if ( arg2 < 0 )
   {
      fprintf( stderr, "%s: index < 0.\n", syntax );
      return 1;
   }

   if ( arg3 < 0 )
   {
      fprintf( stderr, "%s: length < 0.\n", syntax );
      return 1;
   }

   if ( arg2 >= len )
   {
      fprintf( stderr, "%s: index beyond end of string argument.\n", 
               syntax );
      return 1;
   }

   name = str_dup( car1->data.atom->data.string->string,
                   car1->data.atom->data.string->length );

   if ( arg3 && ( arg2 + arg3 ) <= len )
   {
      name[ arg2 + arg3 ] = '\0';
      len = arg3;
   }
   else
      len -= arg2;

   stack_push( stack, make_atom_from_string( &name[ arg2 ], len ));
   free( name );

   return 0;
}

int do_expand( char *syntax, struct object *args )
{
   struct object *car1, *car2; 
   char *ptr;
   int offset = 0, i, len, arg1;
   struct string *s;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   arg1 = *car1->data.atom->data.value;

   ptr = car2->data.atom->data.string->string;
   len = car2->data.atom->data.string->length;

   s = make_string();

   for( i = 0; i < len; ++i )
   {
      if ( *ptr == '\t' )
      {
         int spaces;

         spaces = arg1 - ( i + offset ) % arg1;
         offset += spaces - 1;

         while( spaces-- )
            string_append( s, ' ' );
      }
      else
         string_append( s, *ptr );

      ++ptr;
   }

   stack_push( stack, make_atom_from_string( s->str, s->used ));
   string_free( s );

   return 0;
}

int do_interact( char *syntax, struct object *args )
{
   int i;

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( interactive )
   {
      fprintf( stderr, "%s: already running interactively.\n", syntax );
      return 1;
   }

   interactive = 1;
   i = stack->used;

   for( ; ; )
   {
      struct object *result;
      int depth;

      if ( mode == 0 )
         canon( syntax );

      depth = parse( 0 );

      if ( depth > 0 )
         break;
      else if ( depth < 0 )
         fprintf( stderr, "%d extra ')'\n", -depth );

      result = *( struct object **)stack->top;

      if ( islist( result->flags ) == 0 &&
           result->data.atom->id == stop_id )
         break;

      if ( !evaluate() )
      {
         if ( printer )
         {
            print_object( *( struct object **)stack->top );
            putchar( '\n' );
         }
      }

      thrown = NULL;
      stop = 0;
      next_iteration = 0;
      stack_truncate( stack, stack->used - i );
   }

   stack_truncate( stack, stack->used - i );
   stack_push( stack, make_atom_from_number( 1 ));
   interactive = 0;

   return 0;
}

int do_current( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   stack_push( stack, make_atom_from_string( *arg_ptr, strlen( *arg_ptr )));

   return 0;
}

int do_next( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( arg_ptr < last_arg )
   {
      ++arg_ptr;
      stack_push( stack, make_atom_from_string( *arg_ptr, strlen( *arg_ptr )));
   }
   else
      stack_push( stack, make_atom_from_number( 0 ));

   return 0;
}

int do_prev( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( arg_ptr > first_arg )
   {
      --arg_ptr;
      stack_push( stack, make_atom_from_string( *arg_ptr, strlen( *arg_ptr )));
   }
   else
      stack_push( stack, make_atom_from_number( 0 ));

   return 0;
}

int do_rewind( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   arg_ptr = first_arg;

   stack_push( stack, make_atom_from_string( *arg_ptr, strlen( *arg_ptr )));

   return 0;
}

int do_pwd( char *syntax, struct object *args )
{
   char d[ MAXPATHLEN + 1 ];

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   getcwd( d, sizeof( d ));

   if ( d == NULL )
   {
      fprintf( stderr, "%s: getcwd(): %s.\n", syntax, strerror( errno ));
      return 1;
   }

   stack_push( stack, make_atom_from_string( d, strlen( d )));

   return 0;
}

int do_exit( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   exit( *car->data.atom->data.value );

   return 0;
}

struct object *make_atom_from_table( struct hash_elt **table )
{
   struct object *object;
   struct tree *entry;
   char buffer[ 64 ];

   snprintf( buffer, sizeof( buffer ), "<TABLE#%d>", table_counter++ );

   entry = get_id( buffer, strlen( buffer ));
   entry->flags = ATOM_TABLE;
   entry->data.hash = table;

   object = make_object();
   object->data.atom = entry;
   
   return object;
}

int do_table( char *syntax, struct object *args )
{
   struct hash_elt **table, **ptr;
   int i;

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   table = ( struct hash_elt **)memory( sizeof( struct hash_elt *) * HASH_SIZE );

   ptr = table;

   for( i = 0; i < HASH_SIZE; ++i )
      *ptr++ = NULL;

   stack_push( stack, make_atom_from_table( table ));

   return 0;
}

int do_hash( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3;

   stack_push( arg_stack, ( void *)ERR_TABLE );
   stack_push( arg_stack, ( void *)ERR_ATOM );
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   car3 = stack_pop( stack );
   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   insert_elt( car1->data.atom->data.hash, 
               car2->data.atom->id, car3 );

   stack_push( stack, car3 );

   return 0;
}

int do_unhash( char *syntax, struct object *args )
{
   struct object *car1, *car2;

   stack_push( arg_stack, ( void *)ERR_TABLE );
   stack_push( arg_stack, ( void *)ERR_ATOM );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   remove_elt( car1->data.atom->data.hash, car2->data.atom->id );
   stack_push( stack, car2 );

   return 0;
}

int do_lookup( char *syntax, struct object *args )
{
   struct object *car1, *car2;

   stack_push( arg_stack, ( void *)ERR_TABLE );
   stack_push( arg_stack, ( void *)ERR_ATOM );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   car1 = lookup_elt( car1->data.atom->data.hash, 
                      car2->data.atom->id );

   if ( car1 == NULL )
   {
      stack_push( stack, make_object());
      setlist( ( *( struct object **)stack->top )->flags );
   }
   else
      stack_push( stack, car1 );

   return 0;
}

int do_keys( char *syntax, struct object *args )
{
   struct object *car, **ptr, *result;
   struct stack *keys;
   int idx;

   stack_push( arg_stack, ( void *)ERR_TABLE );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   keys = get_hash_keys( car->data.atom->data.hash );

   result = make_object();
   setlist( result->flags );
   stack_push( stack, result );

   ptr = &result->data.head;

   if ( keys->used == 0 )
      result->data.head = NULL;
   else
   {
      for( idx = 0; idx < keys->used; ++idx )
      {
         *ptr = make_object();
         ( *ptr )->data.atom =  lookup_atom( ( int)keys->values[ idx ] );
         ptr = &( *ptr )->next;
      }
   }

   stack_free( keys );

   return 0;
}

int do_values( char *syntax, struct object *args )
{
   struct object *car, **ptr, *result;
   struct stack *values;
   int idx;

   stack_push( arg_stack, ( void *)ERR_TABLE );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   values = get_hash_values( car->data.atom->data.hash );

   result = make_object();
   setlist( result->flags );
   stack_push( stack, result );

   ptr = &result->data.head;

   if ( values->used == 0 )
      result->data.head = NULL;
   else
   {
      for( idx = 0; idx < values->used; ++idx )
      {
         *ptr = duplicate_object( ( struct object *)values->values[ idx ] );
         ptr = &( *ptr )->next;
      }
   }

   stack_free( values );

   return 0;
}

int do_redirect( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3;
   int flags, fd, arg1;
   mode_t mode;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 1, -1 );
      return 1;
   }

   if ( args->next == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, -1 );
      return 1;
   }

   stack_push( stack, args );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 1, -1 );
      return 1;
   }

   stack_push( stack, args->next );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 2, -1 );
      return 1;
   }

   if ( args->next->next != NULL )
   {
      stack_push( stack, args->next->next );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, 3, -1 );
         return 1;
      }

      car3 = stack_pop( stack );

      if ( islist( car3->flags ) || type( car3->data.atom->flags ) 
           != ATOM_NUMBER )
      {
         print_err( ERR_ARG_TYPE, syntax, 3, ERR_NUMBER );
         return 1;
      }

      if ( args->next->next->next != NULL )
      {
         print_err( ERR_MORE_ARGS, syntax, 3, -1 );
         return 1;
      }
   }
   else
      car3 = NULL;

   car2 = stack_pop( stack );

   if ( islist( car2->flags ) || type( car2->data.atom->flags ) != ATOM_STRING )
   {
      print_err( ERR_ARG_TYPE, syntax, 2, ERR_STRING );
      return 1;
   }

   car1 = stack_pop( stack );

   if ( islist( car1->flags ) || type( car1->data.atom->flags ) != ATOM_NUMBER )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, ERR_NUMBER );
      return 1;
   }

   arg1 = *car1->data.atom->data.value;

   if ( arg1 == 0 && car3 != NULL )
   {
      fprintf( stderr, "%s: append argument given when descriptor is 0.\n",
               syntax );
      return 1;
   }

   if ( arg1 < 0 || arg1 > 2 )
   {
      fprintf( stderr, "%s: descriptor %d out of range.\n", syntax, arg1 );
      return 1;
   }

   flags = 0;

   if ( arg1 )
   {
      flags = O_CREAT;

      if ( car3 != NULL && *car3->data.atom->data.value )
         flags = O_APPEND;
   }

   flags |= ( arg1 == 0 ? O_RDONLY | O_SHLOCK | O_NONBLOCK : 
                          O_WRONLY | O_EXLOCK | O_NONBLOCK );

   mode = getmode( setmode( "0600" ), 0 );

AGAIN:
   if (( fd = open( car2->data.atom->data.string->string, flags, mode )) < 0 )
   {
      if ( errno == EINTR )
         goto AGAIN;

      if ( errno == ENOENT )
      {
         stack_push( stack, make_atom_from_number( -1 ));
         return 0;
      }
      else if ( errno == EACCES )
      {
         stack_push( stack, make_atom_from_number( -2 ));
         return 0;
      }
      else if ( errno == EBUSY || errno == EAGAIN )
      {
         stack_push( stack, make_atom_from_number( -3 ));
         return 0;
      }

      stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));

      return 0;
   }

   if (( arg1 == 1 || arg1 == 2  ) &&
       ( car3 == NULL || *car3->data.atom->data.value == 0 ))
      if ( ftruncate( fd, 0 ))
      {
         if ( errno == EBUSY )
         {
            stack_push( stack, make_atom_from_number( -3 ));
            return 0;
         }

         close( fd );
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }

   if (( flags = dup( arg1 )) < 0 )
   {
      close( fd );
      fprintf( stderr, "%s: dup: %s.\n", syntax, strerror( errno ));
      return 1;
   }

   stack_push( descriptors[ arg1 ], ( void *)flags );

   if ( dup2( fd, arg1 ) < 0 )
   {
      close( fd );
      stack_pop( descriptors[ arg1 ] );
      fprintf( stderr, "%s: dup2: %s.\n", syntax, strerror( errno ));
      return 1;
   }

   close( fd );

   if ( arg1 == 0 )
      getline_from_file( syntax, 1 );

   pipe_pid[ arg1 ] = -1;

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

void resume( char *syntax, int arg1 )
{
   int fd;

   fd = ( int )stack_pop( descriptors[ arg1 ] );

   if ( dup2( fd, arg1 ) < 0 )
   {
      fprintf( stderr, "%s: dup2: %s.\n", syntax, strerror( errno ));
      close( fd );
      return;
   }

   close( fd );

   if ( arg1 == 0 )
      getline_from_file( syntax, 2 );

   pipe_pid[ arg1 ] = -1;
}

int do_resume( char *syntax, struct object *args )
{
   int arg1;
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   arg1 = *car->data.atom->data.value;

   if ( arg1 < 0 || arg1 > 2 )
   {
      fprintf( stderr, "%s: descriptor argument %d out of range.\n", syntax, arg1 );
      return 1;
   }

   if ( descriptors[ arg1 ]->used == 0 )
   {
      stack_push( stack, make_atom_from_number( 0 ));
      return 0;
   }

   resume( syntax, arg1 );
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_warn( char *syntax, struct object *args )
{
   struct object *ptr;
   int i;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   i = 1;

   for( ptr = args; ptr != NULL; ptr = ptr->next )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 1;
      }

      do_print_objects_strings_unquoted( stack_pop( stack ), 0, 2 );

      ++i;
   }

   fputc( '\n', stderr );
   fflush( stderr );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_getenv( char *syntax, struct object *args )
{
   struct object *car;
   char *p;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   p = getenv( car->data.atom->data.string->string );

   if ( p == NULL )
      stack_push( stack, make_atom_from_number( 0 ));
   else
      stack_push( stack, make_atom_from_string( p, strlen( p )));

   return 0;
}

int do_directory( char *syntax, struct object *args )
{
   struct object *car, **ptr, *result;
   DIR *dir;
   struct dirent *dp;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if (( dir = opendir( car->data.atom->data.string->string )) == NULL )
   {
      stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
      return 0;
   }

   readdir( dir );

   result = make_object();
   setlist( result->flags );
   stack_push( stack, result );

   ptr = &result->data.head;

   while(( dp = readdir( dir )) != NULL )
   {
      *ptr = make_atom_from_string( dp->d_name, dp->d_namlen );
      ptr = &( *ptr )->next;
   }

   closedir( dir );

   return 0;
}

int do_chomp( char *syntax, struct object *args )
{
   struct object *car;
   struct string *s;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( car->data.atom->data.string->length )
   {
      int i;

      s = make_string();
      string_assign( s, car->data.atom->data.string->string,
                        car->data.atom->data.string->length );
      i = s->used;

      while( --i >= 0 )
      {
         if ( s->str[ i ] == '\r' || s->str[ i ] == '\n' )
            string_erase( s, i );
         else
            break;
      }

      stack_push( stack, make_atom_from_string( s->str, s->used ));
      string_free( s );
   }
   else
      stack_push( stack, duplicate_object( car ));

   return 0;
}

int do_chop( char *syntax, struct object *args )
{
   char *new;
   int len;
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   new = str_dup( car->data.atom->data.string->string,
                  car->data.atom->data.string->length );
   len = car->data.atom->data.string->length;

   if ( len )
   {
      new[ len - 1 ] = '\0';
      stack_push( stack, make_atom_from_string( new, len - 1 ));
   }
   else
      stack_push( stack, duplicate_object( car ));

   free( new );

   return 0;
}

int do_unlink( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( unlink( car->data.atom->data.string->string ))
   {
      stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
      return 0;
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_rmdir( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   if ( rmdir( car->data.atom->data.string->string ))
   {
      stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
      return 0;
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_rename( char *syntax, struct object *args )
{
   struct object *car1, *car2;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   if ( rename( car1->data.atom->data.string->string, 
                car2->data.atom->data.string->string ) < 0 )
   {
      stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
      return 0;
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_words( char *syntax, struct object *args )
{
   int i, last, result, flag, total;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   if ( buffer->seq( buffer, &dbt_key, &dbt_value, R_LAST ) == -1 )
   {
      fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
      stack_push( stack, make_atom_from_number( 0 ));
   }

   last = *( int *)dbt_key.data;

   if ( !last )
   {
      stack_push( stack, make_atom_from_number( 0 ));
      return 0;
   }

   total = 0;

   for( i = 1; i <= last; ++i )
   {
      char *ptr;

      dbt_key.data = &key_data;
      dbt_key.size = sizeof( recno_t );
      key_data = i;

      if (( result = buffer->get( buffer, &dbt_key, &dbt_value, 0 )) < 0 )
      {
         fprintf( stderr, "%s: db->get: %s.\n", syntax, strerror( errno ));
         return 1;
      }
      else if ( result == 1 )
      {
         fprintf( stderr, "%s: db->get: key does not exist: %d.\n",
                  syntax, i );
         return 1;
      }

      flag = 0;

      ptr = ( char *)dbt_value.data;

      while( *ptr )
      {
         if ( isspace( *ptr++ ))
         {
            if ( flag )
            {
               ++total;
               flag = 0;
            }

            continue;
         }

         flag = 1;
      }
   }

   stack_push( stack, make_atom_from_number( total ));

   return 0;
}

int do_time( char *syntax, struct object *args )
{
   time_t t;

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( time( &t ) < 0 )
   {
      fprintf( stderr, "%s: time(): %s\n.", syntax, strerror( errno ));
      return 1;
   }

   stack_push( stack, make_atom_from_number( t ));

   return 0;
}

int do_random( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   { 
      struct object *obj;
      double r;

      obj = stack_pop( stack );

      if ( *obj->data.atom->data.value <= 0 )
      {
         fprintf( stderr, "%s: argument <= 0: %.10g\n", syntax,
                  *obj->data.atom->data.value );
         return 1;
      }

      r = random();
      r = ( *obj->data.atom->data.value ) * r / RAND_MAX;

      stack_push( stack, make_atom_from_number( r ));
   }

   return 0;
}

int do_date( char *syntax, struct object *args )
{
   struct object *car;
   time_t t;
   struct tm *lt;
   int gmt = 0;
   char buffer[ 64 ];

   if ( args != NULL )
   {
      if ( args->next != NULL )
      {
         print_err( ERR_MORE_ARGS, syntax, 1, -1 );
         return 1;
      }

      stack_push( stack, args );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, 1, -1 );
         return 1;
      }

      car = stack_pop( stack );

      if ( islist( car->flags ) == 1 || 
           type( car->data.atom->flags ) != ATOM_NUMBER )
      {
         print_err( ERR_ARG_TYPE, syntax, 1, ERR_NUMBER );
         return 1;
      }

      gmt = *car->data.atom->data.value;
   }

   if ( time( &t ) < 0 )
   {
      fprintf( stderr, "%s: time(): %s\n.", syntax, strerror( errno ));
      return 1;
   }

   if ( gmt )
   {
      if (( lt = gmtime( &t )) == NULL )
      {
         fprintf( stderr, "%s: gmtime(): %s.\n", syntax, strerror( errno ));
         return 1;
      }
   }
   else
   {
      if (( lt = localtime( &t )) == NULL )
      {
         fprintf( stderr, "%s: localtime(): %s.\n", syntax, strerror( errno ));
         return 1;
      }
   }

   if ( strftime( buffer, sizeof( buffer ) - 1, 
                  "%a, %d %b %Y %H:%M:%S %Z",
                  lt ) == 0 )
   {
      fprintf( stderr, "%s: strftime(): %s.\n", syntax, strerror( errno ));
      return 1;
   }

   stack_push( stack, make_atom_from_string( buffer, strlen( buffer )));

   return 0;
}

int do_when( char *syntax, struct object *args )
{
   struct object *cdr, *result;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   cdr = args->next;

   if ( cdr == NULL )
   {
      fprintf( stderr, "%s: missing body expressions.\n", syntax );
      return 1;
   }

   stack_push( stack, args );

   if ( evaluate() )
   {
      if ( !stop )
         fprintf( stderr, "%s: evaluation of test clause failed.\n", 
                  syntax );

      return 1;
   }

   result = *( struct object **)stack->top;

   if ( !(( islist( result->flags ) == 1 && result->data.head == NULL ) ||
            result->data.atom == zero->data.atom ||
            result->data.atom == empty->data.atom ))
   {
      stack_pop( stack );

      if ( do_progn( syntax, cdr ) )
      {
         if ( !stop )
            fprintf( stderr, "%s: evaluation of subsequent expressions "
                     "failed.\n", syntax );

         return 1;
      }
   }

   return 0;
}

int do_unless( char *syntax, struct object *args )
{
   struct object *cdr, *result;

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   cdr = args->next;

   if ( cdr == NULL )
   {
      fprintf( stderr, "%s: missing body expressions.\n", syntax );
      return 1;
   }

   stack_push( stack, args );

   if ( evaluate() )
   {
      if ( !stop )
         fprintf( stderr, "%s: evaluation of test clause failed.\n", 
                  syntax );

      return 1;
   }

   result = *( struct object **)stack->top;

   if (( islist( result->flags ) == 1 && result->data.head == NULL ) || 
         result->data.atom == zero->data.atom ||
         result->data.atom == empty->data.atom )
   {
      stack_pop( stack );

      if ( do_progn( syntax, cdr ) )
      {
         if ( !stop )
            fprintf( stderr, "%s: evaluation of subsequent expressions "
                     "failed.\n", syntax );

         return 1;
      }
   }

   return 0;
}

int do_test( char *syntax, struct object *args )
{
   struct object *result;

   if ( args == NULL || islist( args->flags ) == 0 )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, ERR_LIST );
      return 1;
   }

   stack_push( stack, args->data.head );

   if ( evaluate() )
   {
      if ( !stop )
         print_err( ERR_EVAL, syntax, 1, 0 );
      return 1;
   }

   result = stack_pop( stack );

   if ( islist( result->flags ) ||
        type( result->data.atom->flags ) != ATOM_MACRO )
   {
      fprintf( stderr, "%s: function position did not evaluate"
                       " to a macro closure.\n", syntax );
      return 1;
   }

   return do_closure( syntax, 
                      result->data.atom->data.closure,
                      args->data.head->next,
                      0 );
}

int do_continue( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   stop = 1;
   next_iteration = 1;
   thrown = NULL;

   return 1;
}

int do_block( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   blocked = 1;
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_unblock( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   blocked = 0;
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_exists( char *syntax, struct object *args )
{
   struct object *car;
   struct stat stats;
   int result;

   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   result = stat( car->data.atom->data.string->string, &stats );

   if ( result < 0 )
   {
      if ( errno == ENOENT )
         stack_push( stack, make_atom_from_number( 0 ));
      else
      {
         fprintf( stderr, "%s: %s.\n", syntax, strerror( errno ));
         return 1;
      }
   }  
   else if ( S_ISREG( stats.st_mode ))
      stack_push( stack, make_atom_from_number( 1 ));
   else if ( S_ISDIR( stats.st_mode ))
      stack_push( stack, make_atom_from_number( 2 ));
   else if ( S_ISCHR( stats.st_mode ))
      stack_push( stack, make_atom_from_number( 3 ));
   else if ( S_ISBLK( stats.st_mode ))
      stack_push( stack, make_atom_from_number( 4 ));
   else if ( S_ISFIFO( stats.st_mode ))
      stack_push( stack, make_atom_from_number( 5 ));
   else if ( S_ISLNK( stats.st_mode ))
      stack_push( stack, make_atom_from_number( 6 ));
   else if ( S_ISSOCK( stats.st_mode ))
      stack_push( stack, make_atom_from_number( 7 ));
   else
      stack_push( stack, make_atom_from_number( 8 ));

   return 0;
}

int do_suspend( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   kill( 0, SIGSTOP );
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_beep( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   beep();

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_length( char *syntax, struct object *args )
{
   struct object *car, *item;
   int i;

   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   i = 0;

   if ( islist( car->flags ))
   {
      for( item = car->data.head; item != NULL; item = item->next )
         ++i;
   }
   else if ( type( car->data.atom->flags ) == ATOM_STRING )
      i = car->data.atom->data.string->length;
   else
   {
      fprintf( stderr, "%s: argument 1 did not evaluate to a string or"
               " a list.\n", syntax );
      return 1;
   }

   stack_push( stack, make_atom_from_number( i ));

   return 0;
}

int do_int( char *syntax, struct object *args )
{
   struct object *car;
   int i;
   double f;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   i = *car->data.atom->data.value;
   f = i;

   stack_push( stack, make_atom_from_number( f ));

   return 0;
}

int do_cos( char *syntax, struct object *args )
{
   struct object *car;
   double r;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   r = cos( *car->data.atom->data.value );
   stack_push( stack, make_atom_from_number( r ));

   return 0;
}

int do_acos( char *syntax, struct object *args )
{
   struct object *car;
   double r;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   r = acos( *car->data.atom->data.value );
   stack_push( stack, make_atom_from_number( r ));

   return 0;
}

int do_tan( char *syntax, struct object *args )
{
   struct object *car;
   double r;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   r = tan( *car->data.atom->data.value );
   stack_push( stack, make_atom_from_number( r ));

   return 0;
}

int do_atan( char *syntax, struct object *args )
{
   struct object *car;
   double r;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   r = atan( *car->data.atom->data.value );
   stack_push( stack, make_atom_from_number( r ));

   return 0;
}

int do_sin( char *syntax, struct object *args )
{
   struct object *car;
   double r;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   r = sin( *car->data.atom->data.value );
   stack_push( stack, make_atom_from_number( r ));

   return 0;
}

int do_asin( char *syntax, struct object *args )
{
   struct object *car;
   double r;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   r = asin( *car->data.atom->data.value );
   stack_push( stack, make_atom_from_number( r ));

   return 0;
}

int do_sqrt( char *syntax, struct object *args )
{
   struct object *car;
   double r;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   r = sqrt( *car->data.atom->data.value );
   stack_push( stack, make_atom_from_number( r ));

   return 0;
}

int do_cbrt( char *syntax, struct object *args )
{
   struct object *car;
   double r;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );

   r = cbrt( *car->data.atom->data.value );
   stack_push( stack, make_atom_from_number( r ));

   return 0;
}

int do_strcmp( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2;
      int i;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      i = strncmp( car1->data.atom->data.string->string,
                   car2->data.atom->data.string->string,
                   MIN( car1->data.atom->data.string->length,
                        car2->data.atom->data.string->length ) + 1 );

      stack_push( stack, make_atom_from_number( i ));
   }

   return 0;
}

int do_fatal( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   fatal = 1;
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_nofatal( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   fatal = 0;
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

#ifdef SQL
int db_busy_handler( void *data, int tries )
{
   struct timespec tv;

	if ( tries == 1000 )
		return 0;

   tv.tv_sec = 0;
   tv.tv_nsec = 250000;

   nanosleep( &tv, NULL );

   return 1;
}

int do_sql_open( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      sqlite3 *db;
      const char *sql_err = NULL;

      car = stack_pop( stack );
      if( sqlite3_open( car->data.atom->data.string->string, &db ))
      {
			sql_err = sqlite3_errmsg( db );
         stack_push( stack, make_atom_from_string( ( char *)sql_err, strlen( sql_err )));
      }
      else
      {
         sqlite3_busy_handler( db, db_busy_handler, NULL );
         stack_push( stack, make_atom_from_db( db ));
      }
   }

   return 0;
}

int do_sql_close( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_DB );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( car->data.atom->data.db == NULL )
         stack_push( stack, make_atom_from_number( 0 ));
      else
      {
         sqlite3_close( car->data.atom->data.db );
         car->data.atom->data.db = NULL;
         stack_push( stack, make_atom_from_number( 1 ));
      }
   }

   return 0;
}

int sql_callback( void *data, int total, char **sql_vals, char **sql_cols )
{
   char **ptr;
   struct object **list_ptr;
   int i;

   if ( sql_list == *( struct object **)stack->top )
   {
      list_ptr = &sql_list->data.head;
      *list_ptr = make_object();
      sql_list = *list_ptr;

      setlist( ( *list_ptr )->flags );
      ( *list_ptr )->next = NULL;
      list_ptr = &( *list_ptr )->data.head;

      ptr = sql_cols;

      for( i = 0; i < total; ++i )
      {
         *list_ptr = make_atom_from_string( *ptr, strlen( *ptr ));
         list_ptr = &( *list_ptr )->next;
         *list_ptr = NULL;
         ++ptr;
      }
   }

   list_ptr = &sql_list->next;
   *list_ptr = make_object();
   sql_list = *list_ptr;

   setlist( ( *list_ptr )->flags );
   ( *list_ptr )->next = NULL;
   list_ptr = &( *list_ptr )->data.head;

   ptr = sql_vals;

   for( i = 0; i < total; ++i )
   {
      *list_ptr = make_atom_from_string( *ptr, strlen( *ptr ));
      list_ptr = &( *list_ptr )->next;
      *list_ptr = NULL;

      ++ptr;
   }

   return 0;
}

int do_sql_exec( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_DB );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *sql, *db;
      char *sql_err;

      sql = stack_pop( stack );
      db = stack_pop( stack );

      if ( db->data.atom->data.db == NULL )
      {
         stack_push( stack, make_atom_from_string( "database has been closed", 24 ));
         return 1;
      }

      sql_list = make_object();
      setlist( sql_list->flags );
      stack_push( stack, sql_list );

      if ( sqlite3_exec( db->data.atom->data.db, 
                         sql->data.atom->data.string->string, 
                         sql_callback, NULL, &sql_err )
           != SQLITE_OK )
      {
         stack_pop( stack );
         stack_push( stack, make_atom_from_string( sql_err, strlen( sql_err )));
         free( sql_err );
      }
   }

   return 0;
}
#endif

#ifdef FCGI
int cgi_printerr( char *syntax )
{
   fprintf( stderr, "%s: FCGI connection is not active.\n", syntax );
   return 1;
}

int do_cgi_accept( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   cgi_in = cgi_out = cgi_err = NULL;
   cgi_param = NULL;

   if ( FCGX_Accept( &cgi_in, &cgi_out, &cgi_err, &cgi_param ) < 0 )
      stack_push( stack, make_atom_from_number( 0 ));
   else
      stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_cgi_finish( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( cgi_out == NULL )
      return cgi_printerr( syntax );

   FCGX_Finish();

   cgi_in = cgi_out = cgi_err = NULL;
   cgi_param = NULL;

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_cgi_xstatus( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   if ( cgi_out == NULL )
      return cgi_printerr( syntax );

   {
      struct object *car;

      car = stack_pop( stack );
      FCGX_SetExitStatus( *car->data.atom->data.value, cgi_out );
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_cgi_getenv( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   if ( cgi_out == NULL )
      return cgi_printerr( syntax );

   {
      struct object *car;
      char *param;

      car = stack_pop( stack );
      param = FCGX_GetParam( car->data.atom->data.string->string, cgi_param );

      if ( param == NULL )
         stack_push( stack, make_atom_from_number( 0 ));
      else
         stack_push( stack, make_atom_from_string( param, strlen( param )));
   }

   return 0;
}

int do_cgi_read( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   if ( cgi_in == NULL )
      return cgi_printerr( syntax );

   {
      struct object *car;
      char *buffer;
      int len;

      car = stack_pop( stack );
      len = *car->data.atom->data.value;

      if ( len < 0 )
      {
         fprintf( stderr, "%s: negative length.\n", syntax );
         return 1;
      }

      if ( len == 0 )
      {
         stack_push( stack, make_atom_from_string( "", 0 ));
         return 0;
      }

      buffer = memory( len + 1 );

      if (( len = FCGX_GetStr( buffer, len, cgi_in )) == 0 )
         stack_push( stack, make_atom_from_number( 0 ));
      else
      {
         buffer[ len ] = '\0';
         stack_push( stack, make_atom_from_string( buffer, len ));
      }

      free( buffer );
   }

   return 0;
}

void cgi_print_objects( struct object *ptr, int recursive )
{
   while( ptr != NULL )
   {
      if ( islist( ptr->flags ) == 1 )
      {
         FCGX_PutChar( '(', cgi_out );
         cgi_print_objects( ptr->data.head, 1 );
         FCGX_PutChar( ')', cgi_out );
      }
      else
      {
         if ( recursive == 0 && type( ptr->data.atom->flags ) == ATOM_STRING )
            FCGX_PutStr( ptr->data.atom->data.string->string,
                         ptr->data.atom->data.string->length,
                         cgi_out );
         else
            FCGX_PutStr( ptr->data.atom->syntax, ptr->data.atom->len, cgi_out );
      }

      if ( recursive == 0 )
         break;

      if (( ptr = ptr->next ) != NULL )
         FCGX_PutChar( ' ', cgi_out );
   }
}

int do_cgi_print( char *syntax, struct object *args )
{
   struct object *ptr;
   int i;

   if ( cgi_out == NULL )
      return cgi_printerr( syntax );

   if ( args == NULL )
   {
      FCGX_FPrintF( cgi_err, "%s: missing arguments.\n", syntax );
      return 1;
   }

   for( i = 1, ptr = args; ptr != NULL; ptr = ptr->next, ++i )
   {
      stack_push( stack, ptr );

      if ( evaluate() )
      {
         if ( !stop )
            FCGX_FPrintF( cgi_err, "%s: error evaluating argument %d.\n", 
                          syntax, i );
         return 1;
      }

      cgi_print_objects( stack_pop( stack ), 0 );
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_cgi_newline( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( cgi_out == NULL )
      return cgi_printerr( syntax );

   FCGX_PutS( "\r\n", cgi_out );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_cgi_warn( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   if ( cgi_err == NULL )
      return cgi_printerr( syntax );

   {
      struct object *car;

      car = stack_pop( stack );

      FCGX_FPrintF( cgi_err, "%s\n", car->data.atom->data.string->string );
      FCGX_FFlush( cgi_err );
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_cgi_sendfile( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      char buffer[ 1024000 ];
      struct object *car;
      int fd, n;

      car = stack_pop( stack );

   AGAIN:
      if (( fd = open( car->data.atom->data.string->string, O_RDONLY )) < 0 )
      {
         if ( errno == EINTR )
            goto AGAIN;

         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }

      for ( n = read( fd, buffer, sizeof( buffer ));
            n > 0; 
            n = read( fd, buffer, sizeof( buffer )) )
         FCGX_PutStr( buffer, n, cgi_out );

      close( fd );

      if ( n < 0 )
      {
         fprintf( stderr, "%s: read: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}
#endif

int do_format( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2, *car3;
      char buffer[ 259 ];
      char format[ 10 ];
      int places, width;

      car3 = stack_pop( stack );
      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      places = *car1->data.atom->data.value;

      if ( places > 10 || places < 0 )
      {
         fprintf( stderr, "%s: argument 1 out of range: %d.\n", 
                  syntax, places );
         return 1;
      }

      width = *car2->data.atom->data.value;

      if ( width < 0 || width > 256 )
      {
         fprintf( stderr, "%s: argument 2 out of range: %d.\n", 
                  syntax, width );
         return 1;
      }

      snprintf( format, sizeof( format ), "%%%d.%df", width, places );
      snprintf( buffer, sizeof( buffer ), format, *car3->data.atom->data.value );

      stack_push( stack, make_atom_from_string( buffer, strlen( buffer )));
   }

   return 0;
}

struct object *make_atom_from_stack( struct stack *stk )
{
   struct object *object;
   struct tree *entry;
   char buffer[ 64 ];

   snprintf( buffer, sizeof( buffer ), "<STACK#%d>", stack_counter++ );

   entry = get_id( buffer, strlen( buffer ));
   entry->flags = ATOM_STACK;
   entry->data.stack = stk;

   object = make_object();
   object->data.atom = entry;
   
   return object;
}

int do_stack( char *syntax, struct object *args )
{
   struct object *result;
   struct stack *stk;
   int i;

   i = 0;

   if ( args != NULL )
   {
      stack_push( stack, args );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, 1, -1 );
         return 1;
      }

      result = stack_pop( stack );

      if ( islist( result->flags ) || 
           type( result->data.atom->flags ) != ATOM_NUMBER )
      {
         print_err( ERR_ARG_TYPE, syntax, 1, ERR_NUMBER );
         return 1;
      }

      i = *result->data.atom->data.value;
      result = NULL;

      if ( i < 0 )
      {
         fprintf( stderr, "%s: initialize size < 0: %d.\n", syntax, i );
         return 1;
      }
   }

   stk = make_stack();
   stack_push( stack, make_atom_from_stack( stk ));

   while( i-- )
   {
      struct object *obj;

      obj = make_object();
      setlist( obj->flags );

      stack_push( stk, obj );
   }

   return 0;
}

int do_pop( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *obj;
      struct stack *stk;

      obj = stack_pop( stack );
      stk = obj->data.atom->data.stack;
      obj = stack_pop( stk );

      if ( obj == NULL )
      {
         obj = make_object();
         setlist( obj->flags );
      }

      stack_push( stack, obj );
   }

   return 0;
}

int do_push( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      stack_push( car1->data.atom->data.stack, car2 );
      stack_push( stack, car2 );
   }

   return 0;
}

int do_unshift( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      int i;
      struct object *car1, *car2;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      if ( car1->data.atom->data.stack->used )
      {
         stack_push( car1->data.atom->data.stack, NULL );

         for( i = car1->data.atom->data.stack->used - 1; i; --i )
            car1->data.atom->data.stack->values[ i ] =
               car1->data.atom->data.stack->values[ i - 1 ];

         car1->data.atom->data.stack->values[ 0 ] = car2;
      }
      else
         stack_push( car1->data.atom->data.stack, car2 );

      stack_push( stack, car2 );
   }

   return 0;
}

int do_shift( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      int i;
      struct object *car, *result;

      car = stack_pop( stack );

      if ( car->data.atom->data.stack->used == 0 )
      {        
         result = make_object();
         setlist( result->flags );
      }
      else
      {
         result = car->data.atom->data.stack->values[ 0 ];

         for( i = 0; i < car->data.atom->data.stack->used - 1; ++i )
            car->data.atom->data.stack->values[ i ] =
               car->data.atom->data.stack->values[ i + 1 ];

         stack_pop( car->data.atom->data.stack );
      }

      stack_push( stack, result );
   }

   return 0;
}

int do_used( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      struct stack *stk;

      car = stack_pop( stack );
      stk = car->data.atom->data.stack;

      stack_push( stack, make_atom_from_number( stk->used ));
   }

   return 0;
}

int do_index( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2;
      struct stack *stk;
      int i;

      car2 = stack_pop( stack );
      i = *car2->data.atom->data.value;

      car1 = stack_pop( stack );
      stk = car1->data.atom->data.stack;

      if ( i < 0 )
      {
         fprintf( stderr, "%s: index < 0: %d.\n", syntax, i );
         return 1;
      }
      else if ( i >= stk->used )
      {
         fprintf( stderr, "%s: index %d out of range.\n", syntax, i );
         return 1;
      }

      stack_push( stack, stk->values[ i ] );
   }

   return 0;
}

int do_store( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2, *car3;
      struct stack *stk;
      int i;

      car3 = stack_pop( stack );

      car2 = stack_pop( stack );
      i = *car2->data.atom->data.value;

      car1 = stack_pop( stack );
      stk = car1->data.atom->data.stack;

      if ( i < 0 || i >= stk->used )
      {
         fprintf( stderr, "%s: index %d out of range.\n", syntax, i );
         return 1;
      }

      stack_push( stack,  stk->values[ i ] = car3 );
   }

   return 0;
}

int do_topidx( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      struct stack *stk;

      car = stack_pop( stack );
      stk = car->data.atom->data.stack;

      stack_push( stack, make_atom_from_number( stk->used - 1 ));
   }

   return 0;
}

int do_extract( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_CLOSURE );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *obj, *result;
      char t;

      obj = stack_pop( stack );
      t = type( obj->data.atom->flags );

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      result->data.head = make_object();

      if ( t == ATOM_CLOSURE )
         result->data.head->data.atom = get_id( "lambda", 6 );
      else
         result->data.head->data.atom = get_id( "macro", 5 );

      result->data.head->next = obj->data.atom->data.closure->text;
   }

   return 0;
}

int do_let( char *syntax, struct object *args )
{
   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, 0 );
      return 1;
   }

   if ( islist( args->flags ) == 0 )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, ERR_LIST );
      return 1;
   }

   if ( args->data.head == NULL )
   {
      fprintf( stderr, "%s: argument 1 is the empty list.\n",
               syntax );
      return 1;
   }

   if ( args->next == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, 0 );
      return 1;
   }

   {
      struct closure *closure;
      struct object *ptr, *sym_list, *arg_list, **ptr2;
      int i;

      sym_list = make_object();
      setlist( sym_list->flags );
      ptr2 = &sym_list->data.head;

      for( i = 1, ptr = args->data.head; ptr!= NULL; ++i, ptr = ptr->next )
      {
         if ( islist( ptr->flags ) == 0 )
         {
            fprintf( stderr, "%s: element %d of argument 1 is not a list.\n",
                     syntax, i );
            return 1;
         }

         if ( ptr->data.head == NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 is the empty list.\n",
                     syntax, i );
            return 1;
         }

         if ( ptr->data.head->next == NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 has only one "
                             "sub-element.\n", syntax, i );
            return 1;
         }

         if ( ptr->data.head->next->next != NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 has more "
                             "than two sub-elements.\n", syntax, i );
            return 1;
         }

         if ( islist( ptr->data.head->flags ) == 1 ||
              type( ptr->data.head->data.atom->flags ) != ATOM_SYMBOL )
         {
            fprintf( stderr, "%s: first sub-element of element %d of argument 1"
                             " is not a symbol.\n", syntax, i );
            return 1;
         }

         *ptr2 = make_object();
         ( *ptr2 )->data.atom = ptr->data.head->data.atom;
         ptr2 = &( *ptr2 )->next;
      }

      *ptr2 = NULL;

      arg_list = make_object();
      setlist( arg_list->flags );
      ptr2 = &arg_list->data.head;

      for( ptr = args->data.head; ptr != NULL; ptr = ptr->next )
      {
         *ptr2 = make_object();
         **ptr2 = *ptr->data.head->next;
         ptr2 = &( *ptr2 )->next;
      }

      *ptr2 = NULL;

      sym_list->next = args->next;

      closure = ( struct closure *)memory( sizeof( struct closure ));
      closure->text = sym_list;
      closure->env = local_env;

      stack_push( stack, arg_list );
      stack_push( stack, make_atom_from_closure( closure, 0 ));

      i = do_closure( syntax, closure, arg_list->data.head, 1 );

      if ( i == 0 )
      {
         ptr = stack_pop( stack );
         stack_pop( stack );
         stack_pop( stack );
         stack_push( stack, ptr );
      }

      return i;
   }
}

int do_letn( char *syntax, struct object *args )
{
   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, 0 );
      return 1;
   }

   if ( islist( args->flags ) == 0 )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, ERR_LIST );
      return 1;
   }

   if ( args->data.head == NULL )
   {
      fprintf( stderr, "%s: argument 1 is the empty list.\n",
               syntax );
      return 1;
   }

   if ( args->next == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, 0 );
      return 1;
   }

   {
      struct object *result, *ptr, **ptr2, **ptr3;
      int i;

      result = NULL;
      ptr2 = &result;

      for( i = 1, ptr = args->data.head; ptr!= NULL; ++i, ptr = ptr->next )
      {
         if ( islist( ptr->flags ) == 0 )
         {
            fprintf( stderr, "%s: element %d of argument 1 is not a list.\n",
                     syntax, i );
            return 1;
         }

         if ( ptr->data.head == NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 is the empty list.\n",
                     syntax, i );
            return 1;
         }

         if ( ptr->data.head->next == NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 has only one "
                             "sub-element.\n", syntax, i );
            return 1;
         }

         if ( ptr->data.head->next->next != NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 has more "
                             "than two sub-elements.\n", syntax, i );
            return 1;
         }

         if ( type( ptr->data.head->data.atom->flags ) != ATOM_SYMBOL )
         {
            fprintf( stderr, "%s: first sub-element of element %d of argument 1"
                             " is not a symbol.\n", syntax, i );
            return 1;
         }

         *ptr2 = make_object();
         setlist( ( *ptr2 )->flags );
         ptr2 = &( *ptr2 )->data.head;

         *ptr2 = make_object();
         ( *ptr2 )->data.atom = get_id( "let", 3 );
         ptr2 = &( *ptr2 )->next;

         *ptr2 = make_object();
         setlist( ( *ptr2 )->flags );
         ptr3 = &( *ptr2 )->next;
         ptr2 = &( *ptr2 )->data.head;

         *ptr2 = make_object();
         setlist( ( *ptr2 )->flags );
         ( *ptr2 )->next = NULL;
         ptr2 = &( *ptr2 )->data.head;

         *ptr2 = ptr->data.head;

         ptr2 = ptr3;
      }

      *ptr2 = args->next;

      stack_push( stack, result );
   }

   return evaluate();
}

int do_labels( char *syntax, struct object *args )
{
   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, 0 );
      return 1;
   }

   if ( islist( args->flags ) == 0 )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, ERR_LIST );
      return 1;
   }

   if ( args->data.head == NULL )
   {
      fprintf( stderr, "%s: argument 1 is the empty list.\n",
               syntax );
      return 1;
   }

   if ( args->next == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, 0 );
      return 1;
   }

   {
      struct object *ptr, *ptr3, *sym_list, *func_list, **ptr2, *result;
      int i;

      sym_list = make_object();
      setlist( sym_list->flags );
      ptr2 = &sym_list->data.head;

      for( i = 1, ptr = args->data.head; ptr!= NULL; ++i, ptr = ptr->next )
      {
         if ( islist( ptr->flags ) == 0 )
         {
            fprintf( stderr, "%s: element %d of argument 1 is not a list.\n",
                     syntax, i );
            return 1;
         }

         if ( ptr->data.head == NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 is the empty list.\n",
                     syntax, i );
            return 1;
         }

         if ( ptr->data.head->next == NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 has only one "
                             "sub-element.\n", syntax, i );
            return 1;
         }

         if ( ptr->data.head->next->next != NULL )
         {
            fprintf( stderr, "%s: element %d of argument 1 has more "
                             "than two sub-elements.\n", syntax, i );
            return 1;
         }

         if ( type( ptr->data.head->data.atom->flags ) != ATOM_SYMBOL )
         {
            fprintf( stderr, "%s: first sub-element of element %d of argument 1"
                             " is not a symbol.\n", syntax, i );
            return 1;
         }

         *ptr2 = make_object();
         ( *ptr2 )->data.atom = ptr->data.head->data.atom;
         ptr2 = &( *ptr2 )->next;
      }

      *ptr2 = NULL;

      func_list = make_object();
      setlist( func_list->flags );

      ptr2 = &func_list->data.head;
      ptr3 = sym_list->data.head;

      for( i = 1, ptr = args->data.head; ptr != NULL; ptr = ptr->next, ++i )

      {
         if ( islist( ptr->data.head->next->flags ) == 0 ||
              ptr->data.head->next->data.head == NULL ||
              islist( ptr->data.head->next->data.head->flags ) == 1 ||
              ( ptr->data.head->next->data.head->data.atom->id != lambda_id &&
                ptr->data.head->next->data.head->data.atom->id != macro_id ))
         {
            fprintf( stderr, "%s: second sub-element of element %d of"
                             " argument 1 is not a lambda or macro expression.\n",
                     syntax, i );
            return 1;
         }

         *ptr2 = make_object();

         setlist( ( *ptr2 )->flags );
         ( *ptr2 )->data.head = make_atom_from_symbol( "set" );

         ( *ptr2 )->data.head->next = make_object();
         setlist( ( *ptr2 )->data.head->next->flags );

         ( *ptr2 )->data.head->next->data.head = 
            make_atom_from_symbol( "quote" );

         ( *ptr2 )->data.head->next->data.head->next =
            make_atom_from_symbol( ptr3->data.atom->syntax );

         ( *ptr2 )->data.head->next->next = ptr->data.head->next;

         ptr2 = &( *ptr2 )->next;
         ptr3 = ptr3->next;
      }

      *ptr2 = args->next;

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      result->data.head = make_object();
      setlist( result->data.head->flags );

      result->data.head->data.head = make_atom_from_symbol( "lambda" );
      result->data.head->data.head->next = sym_list;
      sym_list->next = func_list->data.head;

      ptr2 = &result->data.head->next;

      for( ptr = args->data.head; ptr != NULL; ptr = ptr->next )
      {
         *ptr2 = make_object();
         setlist( ( *ptr2 )->flags );
         ptr2 = &( *ptr2 )->next;
      }
   }

   return evaluate();
}

int do_cond( char *syntax, struct object *args )
{
   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, 0 );
      return 1;
   }

   {
      struct object *ptr, *result = NULL;
      int i;

      for( i = 1, ptr = args; ptr != NULL; ptr = ptr->next, ++i )
      {
         if ( islist( ptr->flags ) == 0 )
         {
            fprintf( stderr, "%s: argument %d is not a list.\n", syntax, i );
            return 1;
         }

         if ( ptr->data.head == NULL )
         {
            fprintf( stderr, "%s: argument %d is the empty list.\n", 
                     syntax, i );
            return 1;
         }

         if ( ptr->data.head->next == NULL )
         {
            fprintf( stderr, "%s: argument %d has only one element.\n",
                     syntax, i );
            return 1;
         }

         stack_push( stack, ptr->data.head );

         if ( evaluate() )
         {
            if ( !stop )
               print_err( ERR_EVAL, syntax, i, 0 );
            return 1;
         }

         result = stack_pop( stack );

         if (( islist( result->flags ) == 1 && result->data.head == NULL ) ||
               result->data.atom == zero->data.atom ||
               result->data.atom == empty->data.atom )
            continue;

         return do_progn( syntax, ptr->data.head->next );
      }

      stack_push( stack, result );
   }

   return 0;
}

int do_transfer( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)5 );

   if ( check_args( syntax, args ))
      return 1;

   if ( buffer_stack->used == 0 )
   {
      fprintf( stderr, "%s: no buffers have been opened.\n", syntax );
      return 1;
   }

   {
      struct object *obj;
      int bn[ 2 ], f1, t1, t2, j, i, inc;
      DB *bp[ 2 ];

      obj = stack_pop( stack );
      t2 = *obj->data.atom->data.value;

      obj = stack_pop( stack );
      bn[ 1 ] = *obj->data.atom->data.value;

      obj = stack_pop( stack );
      t1 = *obj->data.atom->data.value;

      obj = stack_pop( stack );
      f1 = *obj->data.atom->data.value;

      obj = stack_pop( stack );
      bn[ 0 ] = *obj->data.atom->data.value;

      for( j = 0; j < 2; ++j )
      {
         for( i = 0; i < buffer_stack->used; ++i )
         {
            if ( buffer_stack->values[ i ] != NULL )
            {
               if ( i == bn[ j ] )
                  goto CONT;
            }
         }

         fprintf( stderr, "%s: buffer %d is not open.\n", syntax, bn[ j ] );
         return 1;

      CONT:
         bp[ j ] = buffer_stack->values[ i ];
      }

      dbt_key.data = &key_data;
      dbt_key.size = sizeof( recno_t );

      if (( i = buffer->seq( bp[ 0 ], &dbt_key, &dbt_value, R_LAST )) == -1 )
      {
         fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
         return 1;
      }
      else if ( i == 1 )
      {
         fprintf( stderr, "%s: buffer %d is empty.\n", syntax, bn[ 0 ] );
         return 1;
      }

      if ( f1 < 1 || f1 > *( int *)dbt_key.data )
      {
         fprintf( stderr, "%s: argument 2 out of range: %d.\n", syntax, f1 );
         return 1;
      }

      if ( t1 < 1 || t1 > *( int *)dbt_key.data )
      {
         fprintf( stderr, "%s: argument 3 out of range: %d.\n", syntax, t1 );
         return 1;
      }

      if (( i = buffer->seq( bp[ 1 ], &dbt_key, &dbt_value, R_LAST )) == -1 )
      {
         fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
         return 1;
      }
      else if ( i == 1 && t2 != 0 )
      {
         fprintf( stderr, "%s: buffer %d is empty.\n", syntax, bn[ 1 ] );
         return 1;
      }

      if ( t2 < 0 || t2 > *( int *)dbt_key.data )
      {
         fprintf( stderr, "%s: argument 5 out of range: %d.\n", syntax, f1 );
         return 1;
      }

      inc = ( f1 < t1 ? 1 : -1 );

      for( i = f1; ; i += inc )
      {
         key_data = i;
         dbt_key.data = &key_data;
         dbt_key.size = sizeof( recno_t );

         if ( buffer->get( bp[ 0 ], &dbt_key, &dbt_value, 0 ))
         {
            fprintf( stderr, "%s: db->get: %s.\n", syntax, strerror( errno ));
            return 1;
         }

         key_data = t2;

         if ( buffer->put( bp[ 1 ], &dbt_key, &dbt_value, R_IAFTER ))
         {
            fprintf( stderr, "%s: db->put: %s.\n", syntax, strerror( errno ));
            return 1;
         }

         if ( i == t1 )
            break;

         ++t2;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int pipe_open( char *syntax, char *task, int wrt, int std )
{
   int fd[ 2 ], pid, flag;
   char *args[ 4 ];

   if ( pipe( &fd[ 0 ] ) < 0 )
   {
      fprintf( stderr, "%s: pipe: %s.\n", syntax, strerror( errno ));
      return -1;
   }

   switch(( pid = fork() ))
   {
      case -1:
         close( fd[ 0 ] );
         close( fd[ 1 ] );
         fprintf( stderr, "%s: fork: %s.\n", syntax, strerror( errno ));
         return -1;

      case 0:
         if (( dup2( fd[ 1 ], !wrt )) < 0 )
         {
            fprintf( stderr, "%s (child): dup2: %s.\n", syntax, strerror( errno ));
            _exit( 1 );
         }

         close( fd[ 0 ] );
         close( fd[ 1 ] );

         args[ 0 ] = "/bin/sh";
         args[ 1 ] = "-c";
         args[ 2 ] = task;
         args[ 3 ] = NULL;

         execv( args[ 0 ], args );
         _exit( 1 );

      default:
         close( fd[ 1 ] );

         if ( std )
         {
            if (( flag = dup( wrt )) < 0 )
            {
               close( fd[ 0 ] );
               fprintf( stderr, "%s: dup: %s.\n", syntax, strerror( errno ));
               return -1;
            }

            stack_push( descriptors[ wrt ], ( void *)flag );

            if ( dup2( fd[ 0 ], wrt ) < 0 )
            {
               close( fd[ 0 ] );
               close( ( int )stack_pop( descriptors[ wrt ] ));
               fprintf( stderr, "%s: dup2: %s.\n", syntax, strerror( errno ));
               return 1;
            }

            close( fd[ 0 ] );
            pipe_pid[ wrt ] = pid;
         }
     }

   return ( std ? pid : fd[ 0 ] );
}

int do_pipe( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      int arg1, pid;
      struct object *car1, *car2;

      car1 = stack_pop( stack );
      car2 = stack_pop( stack );
      arg1 = *car2->data.atom->data.value;

      if ( arg1 < 0 || arg1 > 2 )
      {
         fprintf( stderr, "%s: argument 1 out of range: %d.\n", syntax, arg1 );
         return 1;
      }

      pid = pipe_open( syntax, car1->data.atom->data.string->string, arg1, 1 );

      if ( pid == -1 )
      {
         stack_push( stack, make_atom_from_number( 0 ));
         return 0;
      }

      if ( arg1 == 0 )
         getline_from_file( syntax, 1 );

      pipe_pid[ arg1 ] = pid;

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_setenv( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      if ( *car1->data.atom->data.string->string == '\0' )
      {
         fprintf( stderr, "%s: argument 1 is empty string.\n", syntax );
         return 1;
      }

      if ( setenv( car1->data.atom->data.string->string, 
                   car2->data.atom->data.string->string, 1 ))
      {
         fprintf( stderr, "%s: setenv: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int lisp_open( char *syntax )
{
   int fd[ 2 ], pid;
   char *args[ 2 ];

   if ( pipe( &fd[ 0 ] ) < 0 )
   {
      fprintf( stderr, "%s: pipe: %s.\n", syntax, strerror( errno ));
      return -1;
   }

   switch(( pid = fork() ))
   {
      case -1:
         close( fd[ 0 ] );
         close( fd[ 1 ] );
         fprintf( stderr, "%s: fork: %s.\n", syntax, strerror( errno ));
         return -1;

      case 0:
         if ( dup2( fd[ 1 ], 0 ) < 0 ||
              dup2( fd[ 1 ], 1 ) < 0 ||
              dup2( fd[ 1 ], 2 ) < 0 )
         {
            fprintf( stderr, "%s (child): dup2: %s.\n", syntax, strerror( errno ));
            _exit( 1 );
         }

         close( fd[ 0 ] );
         close( fd[ 1 ] );

         args[ 0 ] = *first_arg;
         args[ 1 ] = NULL;

         execvp( args[ 0 ], args );
         _exit( 1 );

      default:
         close( fd[ 1 ] );
         lisp_pid = pid;
   }

   return fd[ 0 ];
}

int do_lisp_open( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( lisp_pid != -1 )
   {
      fprintf( stderr, "%s: an inferior lisp is already running.\n", syntax );
      return 1;
   }

   if (( lisp_fd = lisp_open( syntax )) == -1 )
      return 1;

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_lisp_running( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   stack_push( stack, make_atom_from_number( ( lisp_pid == -1 ? 0 : 1 )));

   return 0;
}

int do_lisp_write( char *syntax, struct object *args )
{
   int i;
   struct object *car, *result;

   if ( lisp_pid == -1 )
   {
      fprintf( stderr, "%s: an inferior lisp is not running.\n", syntax );
      return 1;
   }

   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   for( i = 1, car = args; car != NULL; car = car->next, ++i )
   {
      stack_push( stack, car );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, i, -1 );
         return 1;
      }

      result = *( struct object **)stack->top;

      if ( islist( result->flags ) ||
           type( result->data.atom->flags ) != ATOM_STRING )
      {
         print_err( ERR_ARG_TYPE, syntax, i, ERR_STRING );
         return 1;
      }
   }

   {
      char *ptr, *current;
      int len, written;

      while( --i )
      {
         ptr = (( struct object *)stack->values[ stack->used - i ] )->data.atom->data.string->string;
         len = (( struct object *)stack->values[ stack->used - i ] )->data.atom->data.string->length;

         current = ptr;
         written = 0;

         do
         {
            current += written;
            len -= written;

            if (( written = write( lisp_fd, current, len )) < 0 )
            {
               fprintf( stderr, "%s: write: %s.\n", syntax, strerror( errno ));
               return 1;
            }
         }
         while( written < len );
      }

      stack_truncate( stack, i );
      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_lisp_close( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( lisp_fd != -1 )
   {
      close( lisp_fd );
      lisp_fd = -1;
   }

   if ( lisp_pid != -1 )
      kill( lisp_pid, SIGKILL );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_lisp_ready( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( lisp_fd == -1 )
   {
      fprintf( stderr, "%s: an inferior lisp is not running.\n", syntax );
      return 1;
   }  

   {
      int result;
      int fd;
      fd_set in_set;
      struct timeval timeval;

      timeval.tv_sec = 0;
      timeval.tv_usec = 500000;

      fd = lisp_fd;

      FD_ZERO( &in_set );
      FD_SET( fd, &in_set );

      if(( result = select( fd + 1, &in_set, NULL, NULL, &timeval )) < 0 )
         if ( errno != EINTR )
         {
            fprintf( stderr, "%s: select: %s.\n", syntax, strerror( errno ));
            return 1;
         }

      if ( result && FD_ISSET( fd, &in_set ))
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}

int do_lisp_read( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( lisp_fd == -1 )
   {
      fprintf( stderr, "%s: an inferior lisp is not running.\n", syntax );
      return 1;
   }  

   {
      int result;
      int fd;
      fd_set in_set;
      char buffer[ 128 ];
      struct timeval timeval;

      timeval.tv_sec = 30;
      timeval.tv_usec = 0;

      fd = lisp_fd;

      FD_ZERO( &in_set );
      FD_SET( fd, &in_set );

      if(( result = select( fd + 1, &in_set, NULL, NULL, &timeval )) < 0 )
         if ( errno != EINTR )
         {
            fprintf( stderr, "%s: select: %s.\n", syntax, strerror( errno ));
            return 1;
         }

      if ( result && FD_ISSET( fd, &in_set ))
      {
         if (( result = read( fd, buffer, sizeof( buffer ) - 1 )) < 0 )
         {
            fprintf( stderr, "%s: read: %s.\n", syntax, strerror( errno ));
            return 1;
         }

         if ( result == 0 )
         {
            do_lisp_close( syntax, NULL );
            stack_push( stack, make_atom_from_number( 0 ));
         }
         else
         {
            buffer[ result ] = '\0';
            stack_push( stack, make_atom_from_string( buffer, result ));
         }
      }
      else
         stack_push( stack, make_atom_from_string( "", 0 ));
   }

   return 0;
}

int do_protect( char *syntax, struct object *args )
{
   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, -1 );
      return 1;
   }

   {
      struct object *temp, *temp_thrown;
      int temp_stop;

      stack_push( stack, args );

      if ( evaluate() )
      {
         if ( !stop )
            print_err( ERR_EVAL, syntax, 1, -1 );

         temp_thrown = thrown;
         temp_stop = stop;
         temp = NULL;
      }
      else
      {
         temp_thrown = NULL;
         temp_stop = 0;
         temp = stack_pop( stack );
      }

      if ( args->next != NULL && do_progn( syntax, args->next ))
         return 1;

      stop = temp_stop;
      thrown = temp_thrown;

      if ( temp == NULL )
         return 1;

      stack_pop( stack );
      stack_push( stack, temp );
   }

   return 0;
}

int do_tailcall( char *syntax, struct object *args )
{
   struct stack *temp;
   int i;

   if ( current_closure == NULL )
   {
      fprintf( stderr, "%s: no closure is currently being applied.\n",
               syntax );
      return 1;
   }

   temp = make_stack();

   if ( make_act_record( args, current_closure, temp, syntax, 1 ))
      return 1;

   for( i = 1; i < temp->used; i += 2 )
      local_env->data.atom->data.act_record->values[ i ] = temp->values[ i ];

   stack_free( temp );

   stop = 1;
   tailcall = 1;

   return 1;
}

int do_stat( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct stat st;
      struct group *gp;
      struct passwd *pw;
      struct object *obj, *result;

      obj = stack_pop( stack );

      if ( stat( obj->data.atom->data.string->string, &st ) < 0 )
      {
         if ( errno == ENOENT )
         {
            stack_push( stack, make_object());
            setlist( ( *( struct object **)stack->top )->flags );
            return 0;
         }

         fprintf( stderr, "%s: stat: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      pw = getpwuid( st.st_uid );

      if ( pw == NULL )
         obj = make_atom_from_number( st.st_uid );
      else
         obj = make_atom_from_string( pw->pw_name, strlen( pw->pw_name ));

      result->data.head = obj;

      gp = getgrgid( st.st_gid );

      if ( gp == NULL )
         obj->next = make_atom_from_number( st.st_gid );
      else
         obj->next = make_atom_from_string( gp->gr_name, strlen( gp->gr_name ));

      obj->next->next = make_atom_from_number( st.st_atime );
      obj->next->next->next= make_atom_from_number( st.st_mtime );

      obj->next->next->next->next = make_atom_from_number( st.st_size );
   }

   return 0;
}

int do_mkdir( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *obj;
      int mode;

      mode = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;

      obj = ( struct object *)stack_pop( stack );
      mode = mkdir( obj->data.atom->data.string->string, mode );

      if ( mode < 0 )
      {
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_realpath( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *obj;
      char real[ MAXPATHLEN + 1 ];

      obj = stack_pop( stack );

      if ( *obj->data.atom->data.string->string == '\0' )
      {
         stack_push( stack, obj );
         return 0;
      }

      if ( realpath( obj->data.atom->data.string->string, real ) == NULL )
      {
         fprintf( stderr, "%s: realpath: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      stack_push( stack, make_atom_from_string( real, strlen( real )));
   }

   return 0;
}

int do_access( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      int mode;
      struct object *obj1, *obj2;

      obj2 = stack_pop( stack );
      obj1 = stack_pop( stack );
      mode = *obj2->data.atom->data.value;

      if ( mode != 0 && mode != 1 && mode != 2 )
      {
         fprintf( stderr, "%s: unrecognized mode: %d.\n",
                  syntax, mode );
         return 1;
      }

      mode = ( mode ? ( mode == 1 ? W_OK : X_OK ) : R_OK );
      mode = access( obj1->data.atom->data.string->string, mode );

      stack_push( stack, make_atom_from_number( ( mode ? 0 : 1 )));
   }

   return 0;
}

int do_power( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      double x, y, z;
      struct object *obj;

      obj = stack_pop( stack );
      y = *obj->data.atom->data.value;

      obj = stack_pop( stack );
      x = *obj->data.atom->data.value;

      if (( z = pow( x, y )) < 0 )
      {
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 1;
      }

      stack_push( stack, make_atom_from_number( z ));
   }

   return 0;
}

void adjust_bookmarks( int start, int quantity )
{
   int i, line;
   struct stack *keys;

   if ( buffer == NULL )
      return;

   keys = get_hash_keys( bookmarks );

   for( i = 0; i < keys->used; ++i )
   {
      line = ( int )lookup_elt( bookmarks, ( int )keys->values[ i ] );

      if ( line > 0 && line > start )
         insert_elt( bookmarks, 
                     ( int )keys->values[ i ], 
                     ( struct object *)( line + quantity ) );
   }

   stack_free( keys );
}

void delete_bookmarks( int start, int end )
{
   int i, line;
   struct stack *keys;

   if ( buffer == NULL )
      return;

   keys = get_hash_keys( bookmarks );

   for( i = 0; i < keys->used; ++i )
   {
      line = ( int )lookup_elt( bookmarks, ( int )keys->values[ i ] );

      if ( line > 0 && line >= start && line <= end )
         insert_elt( bookmarks, 
                    ( int )keys->values[ i ], 
                    ( struct object *)-1 );
   }

   stack_free( keys );
}

int do_setmark( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_ATOM );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   {
      struct object *car1, *car2;
      int line, last;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      line = *car2->data.atom->data.value;

      if ( do_lastline( syntax, NULL ))
         return 1;

      last = *( *( struct object **)stack->top )->data.atom->data.value;
      stack_pop( stack );

      if ( line < 0 || line > last )
      {
         fprintf( stderr, "%s: line %d out of range.\n", syntax,
                  ( int )*car2->data.atom->data.value );
         return 1;
      }

      insert_elt( bookmarks, car1->data.atom->id, ( struct object *)line );
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_getmark( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_ATOM );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   if ( buffer == NULL )
   {
      fprintf( stderr, "%s: no buffer is open.\n", syntax );
      return 1;
   }

   {
      struct object *car;
      int line;

      car = stack_pop( stack );
      line = ( int )lookup_elt( bookmarks, car->data.atom->id );
      stack_push( stack, make_atom_from_number( line ));
   }

   return 0;
}

#ifdef TERM
void sigwinch_handler( int signo )
{
   struct winsize winsize;

   if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &winsize ) < 0 )
   {
      fprintf( stderr, "sigwinch_handler: %s.\n", strerror( errno ));
      return;
   }

   LINES = winsize.ws_row;
   COLS = winsize.ws_col;

   sigwinch = 1;
}

int do_pause( char *syntax, struct object *args )
{
   struct object *car;
   struct timeval timeval;
   int arg1, fd, c, result, was_canon;
   fd_set in_set;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   arg1 = *car->data.atom->data.value;

   timeval.tv_sec = 0;
   timeval.tv_usec = arg1;

   was_canon = mode;
   nocanon( syntax );

   fd = STDIN_FILENO;

   FD_ZERO( &in_set );
   FD_SET( fd, &in_set );

   if(( result = select( fd + 1, &in_set, NULL, NULL, &timeval )) < 0 )
   {
      if ( errno != EINTR )
      {
         fprintf( stderr, "%s: select: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      stack_push( stack, make_atom_from_number( 1 ));
      return 0;
   }

   if ( result == 0 )
   {
      stack_push( stack, make_atom_from_number( 1 ));
      return 0;
   }

   if ( FD_ISSET( fd, &in_set ))
   {
      c = 0;
      
      if ( read( fd, &c, 1 ) < 0 )
      {
         fprintf( stderr, "%s: read: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      pushed_back = ( int)c;
   }

   if ( was_canon )
      canon( syntax );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_display( char *syntax, struct object *args )
{
   struct object *car1, *car2, *car3;
   int result, i, last, key, end, start, len, tabstop;
   struct string *s;

   if ( ( int )cl == -1 || ( int )ce == -1 || ( int )cm == -1 )
   {
      fprintf( stderr, "%s: terminal has insufficient capabilities.\n", 
               syntax );
      return 1;
   }

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   car3 = stack_pop( stack );
   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   dbt_key.data = &key_data;
   dbt_key.size = sizeof( recno_t );

   if (( result = buffer->seq( buffer, &dbt_key, &dbt_value, R_LAST )) == -1 )
   {
      fprintf( stderr, "%s: db->seq: %s.\n", syntax, strerror( errno ));
      return 1;
   }

   last = *( int *)dbt_key.data;

   key = *car1->data.atom->data.value;

   if ( key < 0 || key > last )
   {
      fprintf( stderr, "%s: line %d out of range.\n", syntax, key );
      return 1;
   }

   start = *car2->data.atom->data.value;

   if ( start < 0 )
   {
      fprintf( stderr, "%s: column %d out of range.\n", syntax, start );
      return 1;
   }

   tabstop = *car3->data.atom->data.value;

   if ( tabstop < 0 )
   {
      fprintf( stderr, "%s: tabstop %d out of range.\n", syntax, tabstop );
      return 1;
   }

   i = 0;
   end = LINES - 1;

   s = make_string();

   if ( key > 0 && last )
      while( key <= last && i < end )
      {
         putp( tgoto( cm, 0, i ));
         putp( ce );
         fflush( stdout );

         dbt_key.data = &key_data;
         dbt_key.size = sizeof( recno_t );
         key_data = key;

         if (( result = buffer->get( buffer, &dbt_key, &dbt_value, 0 )) < 0 )
         {
            fprintf( stderr, "%s: db->get: %s.\n", syntax, strerror( errno ));
            string_free( s );
            return 1;
         }
         else if ( result == 1 )
         {
            fprintf( stderr, "%s: db->get: key does not exist.\n", syntax );
            string_free( s );
            return 1;
         }

         len = strlen( dbt_value.data );

         if ( start < len )
         {
            int j, limit, offset;
            char *ptr;

            string_truncate( s );

            ptr = ( char *)dbt_value.data;
            len = strlen( ptr );
            offset = 0;

            for( j = 0; j < len; ++j )
            {
               if ( *ptr == '\t' )
               {
                  int spaces;

                  spaces = tabstop - ( j + offset ) % tabstop;
                  offset += spaces - 1;

                  while( spaces-- )
                     string_append( s, ' ' );
               }
               else if ( *ptr != '\r' && *ptr != '\n' )
                  string_append( s, *ptr );

               ++ptr;
            }

            limit = MIN( s->used , start + COLS );
            s->str[ limit ] = '\0';
            fputs( &s->str[ start ], stdout );
         }

         ++key;
         ++i;
      }

   while( i < end )
   {
      putp( tgoto( cm, 0, i ));
      putp( ce );
      putchar( '~' );
      ++i;
   }

   fflush( stdout );

   string_free( s );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_scrollup( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( ( int )sf == -1 || ( int )sc == -1 )
   {
      fprintf( stderr, "%s: terminal has insufficient capabilities.\n", 
               syntax );
      return 1;
   }
   
   putp( sc );
   putp( tgoto( cm, 0, LINES - 1 ));
   putp( sf );
   putp( rc );
   fflush( stdout );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_scrolldn( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( ( int )sr == -1 || ( int )sc == -1 )
   {
      fprintf( stderr, "%s: terminal has insufficient capabilities.\n", 
               syntax );
      return 1;
   }

   putp( sc );
   putp( tgoto( cm, 0, 0 ));
   putp( sr );
   putp( rc );
   fflush( stdout );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_clearscreen( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( ( int)cl != -1 )
   {
      putp( cl );
      fflush( stdout );
   }

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_goto( char *syntax, struct object *args )
{
   struct object *car1, *car2;
   int y, x;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   y = *car1->data.atom->data.value;
   x = *car2->data.atom->data.value;

   if ( y < 0 || y > LINES )
   {
      fprintf( stderr, "%s: line index %d out of range.\n", 
               syntax, y );
      return 1;
   }

   if ( x < 0 || x > COLS )
   {
      fprintf( stderr, "%s: column index %d out of range.\n", 
               syntax, x );
      return 1;
   }

   putp( tgoto( cm, x, y ));

   fflush( stdout );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_clearline( char *syntax, struct object *args )
{
   struct object *car1, *car2;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   car2 = stack_pop( stack );
   car1 = stack_pop( stack );

   if ( ( *car1->data.atom->data.value ) < 0 || 
        ( *car1->data.atom->data.value ) > LINES - 1 )
   {
      fprintf( stderr, "%s: line %d out of range.\n", syntax, 
               ( int )*car1->data.atom->data.value );
      return 1;
   }

   if ( ( *car2->data.atom->data.value ) < 0 || 
        ( *car2->data.atom->data.value ) > COLS - 1 )
   {
      fprintf( stderr, "%s: column %d out of range.\n", syntax, 
               ( int )*car2->data.atom->data.value );
      return 1;
   }

   putp( tgoto( cm, *car2->data.atom->data.value, 
                    *car1->data.atom->data.value ));
   putp( ce );
   fflush( stdout );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_hide( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   putp( vi );
   fflush( stdout );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_show( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   putp( ve );
   fflush( stdout );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_insertln( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( ( int )al == - 1 )
   {
      fprintf( stderr, "%s: terminal has insufficient capabilities.\n", 
               syntax );
      return 1;
   }

   putp( al );
   fflush( stdout );

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_getchar( char *syntax, struct object *args )
{
   int result, c, was_canon;

   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   if ( pushed_back >= 0 )
   {
      stack_push( stack, make_atom_from_number( pushed_back ));
      pushed_back = -1;
      return 0;
   }

   was_canon = mode;

AGAIN:
   nocanon( syntax );

   c = 0;
   result = read( STDIN_FILENO, &c, 1 );

   if ( was_canon )
      canon( syntax );

   if ( result == 0 )
      stack_push( stack, make_atom_from_number( -1 ));
   else if ( result < 0 )
   {
      if ( errno == EINTR )
      {
         if ( !sigwinch )
            goto AGAIN;
         else
         {
            stack_push( stack, make_atom_from_number( -2 ));
            sigwinch = 0;
         }
      }
      else
      {
         fprintf( stderr, "%s: %s.\n", syntax, strerror( errno ));
         return 1;
      }
   }
   else
      stack_push( stack, make_atom_from_number( c ));

   return 0;
}

int do_pushback( char *syntax, struct object *args )
{
   struct object *car;

   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   car = stack_pop( stack );
   pushed_back = *car->data.atom->data.value;

   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_canon( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *) 0 );

   if ( check_args( syntax, args ))
      return 1;

   canon( syntax );
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_nocanon( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   nocanon( syntax );
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}
#endif

int do_noprinter( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   printer = 0;

   return 0;
}

int do_printer( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   printer = 1;
   stack_push( stack, make_atom_from_number( 1 ));

   return 0;
}

int do_exec( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      char *exec_args[ 4 ];

      car = stack_pop( stack );

      exec_args[ 0 ] = "/bin/sh";
      exec_args[ 1 ] = "-c";
      exec_args[ 2 ] = car->data.atom->data.string->string;
      exec_args[ 3 ] = NULL;

      execv( exec_args[ 0 ], exec_args );
      
      fprintf( stderr, "%s: execv: %s.\n", syntax, strerror( errno ));
   }

   return 1;
}

int do_truncate( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1;

      car1 = stack_pop( stack );

      if ( ftruncate( STDOUT_FILENO, *car1->data.atom->data.value ) < 0 )
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
      else
         stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_symbolp( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( islist( car->flags ) == 0 && type( car->data.atom->flags ) == ATOM_SYMBOL )
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}

int do_regexpp( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( islist( car->flags ) == 0 && type( car->data.atom->flags ) == ATOM_REGEXP )
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}

int do_tablep( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( islist( car->flags ) == 0 && type( car->data.atom->flags ) == ATOM_TABLE )
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}

int do_stackp( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( islist( car->flags ) == 0 && type( car->data.atom->flags ) == ATOM_STACK )
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}

int do_intrinsicp( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( islist( car->flags ) == 0 && type( car->data.atom->flags ) == ATOM_INTRINSIC )
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}

int do_closurep( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( islist( car->flags ) == 0 && type( car->data.atom->flags ) == ATOM_CLOSURE )
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}

int do_macrop( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)-1 );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;

      car = stack_pop( stack );

      if ( islist( car->flags ) == 0 && type( car->data.atom->flags ) == ATOM_MACRO )
         stack_push( stack, make_atom_from_number( 1 ));
      else
         stack_push( stack, make_atom_from_number( 0 ));
   }

   return 0;
}

int do_dynamic_let( char *syntax, struct object *args )
{
   if ( args == NULL )
   {
      print_err( ERR_MISSING_ARGS, syntax, 0, 0 );
      return 1;
   }

   if ( islist( args->flags ) == 0 )
   {
      print_err( ERR_ARG_TYPE, syntax, 1, ERR_LIST );
      return 1;
   }

   if ( args->data.head == NULL )
   {
      fprintf( stderr, "%s: argument 1 is the empty list.\n",
               syntax );
      return 1;
   }

   if ( args->next == NULL )
   {
      print_err( ERR_MISSING_ARG, syntax, 2, 0 );
      return 1;
   }

   if ( args->data.head == NULL )
   {
      fprintf( stderr, "%s: argument 1 is the empty list.\n", syntax );
      return 1;
   }

   if ( args->data.head->next == NULL )
   {
      fprintf( stderr, "%s: element 1 of argument 1 contains only one sub-element.\n", syntax );
      return 1;
   }

   if ( args->data.head->next->next != NULL )
   {
      fprintf( stderr, "%s: element 1 of argument 1 contains more than two sub-elements.\n", syntax );
      return 1;
   }

   if ( islist( args->data.head->flags ) == 1 ||
        type( args->data.head->data.atom->flags ) != ATOM_SYMBOL )
   {
      fprintf( stderr, "%s: element 1 of argument 1 is not a symbol.\n", syntax );
      return 1;
   }

   {
      struct object *value, *old;
      int result;

      stack_push( stack, args->data.head->next );

      if ( evaluate() )
      {
         if ( !stop )
            fprintf( stderr, "%s: evaluation of expression to bind failed.\n", syntax );

         return 1;
      }

      value = stack_pop( stack );
      old = lookup_binding( args->data.head->data.atom->id );
      insert_binding( args->data.head->data.atom->id, value );

      if ( old != NULL )
         stack_push( stack, old );

      result = do_progn( syntax, args->next );

      if ( old != NULL )
         insert_binding( args->data.head->data.atom->id, old );
      else
         remove_binding( args->data.head->data.atom->id );

      if ( result == 0 && old != NULL )
      {
         struct object *returned;

         returned = stack_pop( stack );
         stack_pop( stack );
         stack_push( stack, returned );
      }

      return result;
   }
}

int do_chmod( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );
   
   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2;
      struct stat st;
      mode_t newmode;
      void *mode;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      mode = setmode( car1->data.atom->data.string->string );

      if ( mode == NULL )
      {
         char buffer[ 256 ];

         snprintf( buffer, sizeof( buffer ), "Invalid mode: %s", 
                   car1->data.atom->data.string->string );
         stack_push( stack, make_atom_from_string( buffer, strlen( buffer )));
         return 0;
      }

      if ( stat( car2->data.atom->data.string->string, &st ) < 0 )
      {
         free( mode );
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }

      newmode = getmode( mode, st.st_mode );
      free( mode );

      if ( chmod( car2->data.atom->data.string->string, newmode ) < 0 )
      {
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 1;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_chown( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   {
      char buffer[ 256 ];
      struct object *car1, *car2, *car3;
      int gid, uid;

      car3 = stack_pop( stack );
      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      gid = uid = -1;

      if ( car1->data.atom->data.string->length )
      {
         struct passwd *pass;

         if ( getuid() != 0 && geteuid() != 0 )
         {
            stack_push( stack, make_atom_from_string( "only root may change user", 25 )); 
            return 0;
         }

         pass = getpwnam( car1->data.atom->data.string->string );

         if ( pass == NULL )
         {
            snprintf( buffer, sizeof( buffer ), "no such user: %s", 
                      car1->data.atom->data.string->string );
            stack_push( stack, make_atom_from_string( buffer, strlen( buffer )));
            return 0;
         }

         uid = pass->pw_uid;
      }

      if ( car2->data.atom->data.string->length == 0 || uid == -1 )
      {
         struct stat st;

         if ( stat( car3->data.atom->data.string->string, &st ) < 0 )
         {
            stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
            return 0;
         }

         if ( uid == -1 )
            uid = st.st_uid;
         else
            gid = st.st_gid;
      }

      if ( car2->data.atom->data.string->length )
      {
         struct group *group;

         if (( group = getgrnam( car2->data.atom->data.string->string )) == NULL )
         {
            snprintf( buffer, sizeof( buffer ), "no such group: %s", 
                      car2->data.atom->data.string->string );
            stack_push( stack, make_atom_from_string( buffer, strlen( buffer )));
            return 0;
         }

         gid = group->gr_gid;
      }

      if ( chown( car3->data.atom->data.string->string, uid, gid ) < 0 )
      {
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_clear( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STACK );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2;
      int n;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      n = *car2->data.atom->data.value;

      if ( n < 0 )
      {
         fprintf( stderr, "%s: argument 2 is negative number.\n", syntax );
         return 1;
      }

      while( n-- && car1->data.atom->data.stack->used )
         stack_pop( car1->data.atom->data.stack );

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_basename( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void*)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      char *ptr;

      car = stack_pop( stack );

      if (( ptr = basename( car->data.atom->data.string->string )) == NULL )
         stack_push( stack, make_atom_from_string( "", 0 ));
      else 
         stack_push( stack, make_atom_from_string( ptr, strlen( ptr )));
   }

   return 0;
}

int do_dirname( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void*)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      char *ptr;

      car = stack_pop( stack );

      if (( ptr = dirname( car->data.atom->data.string->string )) == NULL )
         stack_push( stack, make_atom_from_string( "", 0 ));
      else 
         stack_push( stack, make_atom_from_string( ptr, strlen( ptr )));
   }

   return 0;
}

int do_checkpass( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2;
      struct passwd *passwd;
      char *encrypted;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      if (( passwd = getpwnam( car1->data.atom->data.string->string )) == NULL )
      {
         stack_push( stack, make_atom_from_number( 0 ));
         return 0;
      }

      if (( encrypted = crypt( car2->data.atom->data.string->string, 
                               passwd->pw_passwd )) == NULL )
      {
         stack_push( stack, make_atom_from_number( 0 ));
         return 0;
      }

      if ( strcmp( passwd->pw_passwd, encrypted ))
      {
         stack_push( stack, make_atom_from_number( 0 ));
         return 0;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_setuid( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      struct passwd *passwd;

      car = stack_pop( stack );

      if (( passwd = getpwnam( car->data.atom->data.string->string )) == NULL )
      {
         stack_push( stack, make_atom_from_number( 0 ));
         return 0;
      }

      stack_push( stack, make_atom_from_number( setuid( passwd->pw_uid ) ? 0 : 1 ));
   }

   return 0;
}

int do_getuid( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *result;
      struct passwd *passwd;

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      if (( passwd = getpwuid( getuid() )) == NULL )
         return 0;

      result->data.head = make_atom_from_string( passwd->pw_name, strlen( passwd->pw_name ));
      result->data.head->next = make_atom_from_number( passwd->pw_uid );
   }
   
   return 0;
}

int do_getgid( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *result;
      struct group *gp;

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      if (( gp = getgrgid( getgid() )) == NULL )
         return 0;

      result->data.head = make_atom_from_string( gp->gr_name, strlen( gp->gr_name ));
      result->data.head->next = make_atom_from_number( gp->gr_gid );
   }

   return 0;
}

int do_geteuid( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *result;
      struct passwd *passwd;

      result = make_object();
      setlist( result->flags );
      stack_push( stack, result );

      if (( passwd = getpwuid( geteuid() )) == NULL )
         return 0;

      result->data.head = make_atom_from_string( passwd->pw_name, strlen( passwd->pw_name ));
      result->data.head->next = make_atom_from_number( passwd->pw_uid );
   }

   return 0;
}

int do_seteuid( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      struct passwd *passwd;

      car = stack_pop( stack );

      if (( passwd = getpwnam( car->data.atom->data.string->string )) == NULL )
      {
         stack_push( stack, make_atom_from_number( 0 ));
         return 0;
      }

      stack_push( stack, make_atom_from_number( seteuid( passwd->pw_uid ) ? 0 : 1 ));
   }

   return 0;
}

int do_seek( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)3 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2, *car3;
      char *w;
      int fd, offset, whence;

      car3 = stack_pop( stack );
      car2 = stack_pop( stack );
      car1 = stack_pop( stack );
      
      if (( fd = *car1->data.atom->data.value ) != 0 &&
            fd != 1 )
      {
         fprintf( stderr, "%s: descriptor argument out of range: %d.\n", syntax, fd );
         return 1;
      }

      offset = *car2->data.atom->data.value;
      w = car3->data.atom->data.string->string;

      if ( strcmp( w, "SEEK_SET") == 0 )
         whence = SEEK_SET;
      else if ( strcmp( w, "SEEK_CUR" ) == 0 )
         whence = SEEK_CUR;
      else if ( strcmp( w, "SEEK_END" ) == 0 )
         whence = SEEK_END;
      else
      {
         fprintf( stderr, "%s: unrecognized whence argument: %s.\n", syntax, w );
         return 1;
      }

      if (( offset = lseek( fd, offset, whence )) < 0 )
      {
         fprintf( stderr, "%s: lseek: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      stack_push( stack, make_atom_from_number( offset ));
   }

   return 0;
}

int do_getchars( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      char *buffer, *ptr;
      int total, returned;

      car = stack_pop( stack );
      total = *car->data.atom->data.value;
      
      if ( total == 0 )
      {
         stack_push( stack, make_atom_from_string( "", 0 ));
         return 0;
      }

      if ( total < 0 )
      {
         fprintf( stderr, "%s: argument 1 < 0.\n", syntax );
         return 1;
      }

      buffer = memory( total + 1 );
      ptr = buffer;

AGAIN:
      if (( returned = read( STDIN_FILENO, ptr, total )) < 0 )
      {
         if ( errno == EINTR )
            goto AGAIN;

         free( buffer );
         fprintf( stderr, "%s: read: %s.\n", syntax, strerror( errno ));
         return 1;
      }

      if ( returned == 0 )
      {
         if ( ptr != buffer )
         {
            *ptr = '\0';
            stack_push( stack, make_atom_from_string( buffer,  ptr - buffer ));
         }
         else
            stack_push( stack, make_atom_from_number( 0 ));

         free( buffer );
         return 0;
      }

      if ( returned < total )
      {
         ptr = &ptr[ returned ];
         total -= returned;
         goto AGAIN;
      }

      returned += ptr - buffer;
      buffer[ returned ] = '\0';

      stack_push( stack, make_atom_from_string( buffer, returned ));
      free( buffer );
   }

   return 0;
}

int do_readlock( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      int result;

      if (( result = flock( STDIN_FILENO, LOCK_SH | LOCK_NB )) < 0 )
      {
         if ( errno == EWOULDBLOCK )
            stack_push( stack, make_atom_from_number( 0 ));
         else
            stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_writelock( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      int result;

      if (( result = flock( STDOUT_FILENO, LOCK_EX | LOCK_NB )) < 0 )
      {
         if ( errno == EWOULDBLOCK )
            stack_push( stack, make_atom_from_number( 0 ));
         else
            stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_unlock( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_NUMBER );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car;
      int result, fd;

      car = stack_pop( stack );
      fd = *car->data.atom->data.value;

      if ( fd != 0 && fd != 1 )
      {
         fprintf( stderr, "%s: invalid descriptor: %d.\n", syntax, fd );
         return 1;
      }

      if (( result = flock( fd, LOCK_UN | LOCK_NB )) < 0 )
      {
         if ( errno == EWOULDBLOCK )
            stack_push( stack, make_atom_from_number( 0 ));
         else
            stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }

      stack_push( stack, make_atom_from_number( 1 ));
   }

   return 0;
}

int do_hostname( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)0 );

   if ( check_args( syntax, args ))
      return 1;

   {
      char hostname[ MAXHOSTNAMELEN ];

      if ( gethostname( hostname, MAXHOSTNAMELEN ) < 0 )
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
      else
         stack_push( stack, make_atom_from_string( hostname, strlen( hostname )));
   }

   return 0;
}

int do_symlink( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)2 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct object *car1, *car2;

      car2 = stack_pop( stack );
      car1 = stack_pop( stack );

      if ( symlink( car1->data.atom->data.string->string,
                    car2->data.atom->data.string->string ) < 0 )
      {
         stack_push( stack, make_atom_from_string( strerror( errno ), -1 ));
         return 0;
      }
   }

   stack_push( stack, make_atom_from_number( 1 ));
   return 0;
}

int do_gecos( char *syntax, struct object *args )
{
   stack_push( arg_stack, ( void *)ERR_STRING );
   stack_push( arg_stack, ( void *)1 );

   if ( check_args( syntax, args ))
      return 1;

   {
      struct passwd *passwd;
      struct object *car;

      car = stack_pop( stack );

      if (( passwd = getpwnam( car->data.atom->data.string->string )) == NULL )
      {
         stack_push( stack, make_atom_from_string( "", 0 ));
         return 0;
      }

      stack_push( stack, make_atom_from_string( passwd->pw_gecos, strlen( passwd->pw_gecos )));
   }

   return 0;
}
